//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.

import axios from 'axios';
import properties from '../../config/dashboardProperties';

const host = properties.wrapperAPI.host;
const port = properties.wrapperAPI.port;

/*-------------config-------------*/
const config ={
    //baseURL: 'http://'+host+':'+port+'/api/v1/',
    baseURL: 'http://'+host+'/api/v1/',
    timeout: 10000,
    headers: {'Accept': 'application/json'}
  };

axios.defaults.headers.post['Content-Type'] = 'application/x-www-form-urlencoded';
/*-----------------------------------*/

export const getStatus = () => axios.get('status',config)
.then((response) => {
    return response;})
.catch((err) => {
  if (!err.response) {
              // network error
            console.log('Error: Network Error. Unable to connect to '+ host+":"+port);
  } else {
            console.log(err.response.data.message);
  }
    return err;
});

export const getAllDataset = () => axios.get('getAllDataset/',config)
.then((response) => {
    return response;})
.catch((err) => {
  if (!err.response) {
              // network error
            console.log('Error: Network Error. Unable to connect to '+ host+":"+port);
  } else {
            console.log(err.response.data.message);
  }
    return err;
});

export const getDataset = (parameters) =>
  axios.get('getDataset/',{...config, params : parameters})
.then((response) => {
  console.log(parameters)

    return response;})
.catch((err) => {
  console.log(parameters)

  if (!err.response) {
              // network error
            console.log('Error: Network Error. Unable to connect to '+ host+":"+port);
  } else {
            console.log(err.response.data.message);
  }
    return err;
});

export const removeDataset = (datasetID) =>
  axios.get('removeDataset/',{...config, params : {"datasetID": datasetID}})
.then((response) => {
    return response;})
.catch((err) => {
  if (!err.response) {
              // network error
            console.log('Error: Network Error. Unable to connect to '+ host+":"+port);
  } else {
            console.log(err.response.data.message);
  }
    return err;
});

export const addDataset = (payLoad) =>

  axios.post('addDataset', payLoad, config)
  .then((response) => {
      return response;})
  .catch((err) => {
    if (!err.response) {
                // network error
              console.log('Error: Network Error. Unable to connect to '+ host+":"+port);
    } else {
              console.log(err.response.data.message);
    }
      return err;
  });

  export const updateDataset = (request,datasetID) => {
    //let request = {
    //    "lastAssessmentRequestID" : payLoad,
    //}

  return  axios.post('updateDataset', request, {...config, params : {"datasetID": datasetID}})
      .then((response) => {
          return response;})
      .catch((err) => {
        if (!err.response) {
                    // network error
                  console.log('Error: Network Error. Unable to connect to '+ host+":"+port);
        } else {
                  console.log(err.response.data.message);
        }
          return err;
      });
  }


  export const generateKnowledgeBase = (metrics, qualityGraph, dataGraph, lastAssessmentDate,isSparqlEndPoint, knowledgeBaseMapping) => {

let request = {
  "metrics" : metrics,
  "qualityGraph" : qualityGraph,
  "dataGraph" : dataGraph,
  "tripleStoreEndPoint" : "http://"+properties.tripleStore.host+":"+properties.tripleStore.port+"/"+properties.tripleStore.datastore+"/query",
  "lastAssessmentDate" : lastAssessmentDate,
  "isSparqlEndPoint" : isSparqlEndPoint.toString(),
  "knowledgeBaseMapping": knowledgeBaseMapping
}
  return  axios.post('knowledgeBase', request, config)
      .then((response) => {
          return response;})
      .catch((err) => {
        if (!err.response) {
                    // network error
                  console.log('Error: Network Error. Unable to connect to '+ host+":"+port);
        } else {
                  console.log(err.response.data.message);
        }
          return err;
      });
  }

  export const getKnowledgeBaseStatus = (knowledgeBaseID) =>
    axios.get('knowledgeBase',{...config, params : {"knowledgeBaseID": knowledgeBaseID}})
  .then((response) => {
      return response;})
  .catch((err) => {
    if (!err.response) {
                // network error
              console.log('Error: Network Error. Unable to connect to '+ host+":"+port);
    } else {
              console.log(err.response.data.message);
    }
      return err;
  });

  export const getKnowledgeBase= (knowledgeBaseID) =>
    axios.get('getKnowledgeBase',{...config, params : {"knowledgeBaseID": knowledgeBaseID}})
  .then((response) => {
      return response;})
  .catch((err) => {
    if (!err.response) {
                // network error
              console.log('Error: Network Error. Unable to connect to '+ host+":"+port);
    } else {
              console.log(err.response.data.message);
    }
      return err;
  });

export default {getAllDataset,getDataset,removeDataset,addDataset, updateDataset, generateKnowledgeBase, getKnowledgeBaseStatus, getKnowledgeBase, getStatus};
