//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component } from 'react';
import { connect } from 'react-redux';
import { Row, Col, Dropdown, DropdownToggle, DropdownMenu, DropdownItem, InputGroup, Input } from 'reactstrap';
import ConnectDataSet from './ConnectDataSet/ConnectDataSet';
import DataSetLarge from './DataSetLarge/DataSetLarge'
import { getAllDataset, removeDataset,getDataset } from '../../services/datasetConfigDetails/datasetConfigDetails';
import { deleteGraph } from '../../services/tripleStoreAPIs/deleteGraphFromTripleStore';
import DatasetConnectConfigModelWindow from './ConnectDataSet/DatasetConnectConfigModelWindow';
import DatasetConnectOptionsModelWindow from './ConnectDataSet/DatasetConnectOptionsModelWindow';
import LoadingSpinner from './loading';
import LoadingFailed from './loadingFailed';
import properties from '../../config/dashboardProperties';
import TripleStoreUploadError from './tripleStoreUploadError';

const limit = properties.dataQualityOverTimeBarCount;

class Dashboard extends Component {

  constructor(props) {
    super(props);
    if (sessionStorage.getItem("LIMIT")===null) {
      sessionStorage.setItem("LIMIT",limit );
    }

  }

    state = {
        datasetDetails: [],
        dropdownOpen: false,
        selectedView: "Large Title View",
        searchString: "",
        showConfigModal: false,
        showOptionModel: false,
        connectDataSet: null
    };


    toggleDropDown = () => {
        this.setState(prevState => ({
            dropdownOpen: !prevState.dropdownOpen
        }));
    }

    toggleAlertModel = () => {
        this.setState({
            loadFailed: false
        });
    }

    toggleBetweenConfigAndOptionModel = (modelState) =>{
      this.setState(prevState => ({
          showConfigModal: !prevState.showConfigModal,
          showOptionModel: !prevState.showOptionModel,
          connectDataSet : modelState
      }));
    }

    toggleConfigModel = () => {
      if(this.state.showConfigModal){
        //If it is already open, then close it and set connectDataSet as null
        this.setState(prevState => ({
            showConfigModal: !prevState.showConfigModal,
            connectDataSet : null
        }));
      }
      else {
        this.setState(prevState => ({
            showConfigModal: !prevState.showConfigModal
        }));
      }
    }

    toggleOptionModel = (uploadError,graphName) => {
      //console.log(uploadError);
      if(this.state.showOptionModel){
        //If it is already open, then close it and set connectDataSet as null
        this.setState(prevState => ({
            showOptionModel: !prevState.showOptionModel,
            connectDataSet : null
        }));
      }
      else {
        this.setState(prevState => ({
            showOptionModel: !prevState.showOptionModel
        }));
      }

      if(!uploadError)
      {
        this.setState({
          showTripleStoreError:true,
          graphName:graphName
        });
      }

    }

    toggleTripleStoreErrorModel = () => {
        this.setState({
            showTripleStoreError: false,
            graphName:""
        });
    }

    componentWillUnmount(){
    }

    componentDidMount() {
      //If there are no datasetDetails in Cache, then Call API to fetch the data
      //and the update the sessionStorage Cache.

      if(this.props.datasetDetailsCache){
        this.setState({
            datasetDetails: [...this.props.datasetDetailsCache]
        });
      }
    }

    componentDidUpdate() {

    }

    removeDataSetComponent = (dataSetID, graphName) => {
        //This function will remove delete the DataSet details from the dataset config file
        this.setState({ isloading: true }, () =>{
          removeDataset(dataSetID).then((response) => {
          if (response.status === 200) {
            this.setState({
                isloading: false,
                //loadFailed: false
            }, async ()=>{
              await this.props.removeDatasetDetailsCache(dataSetID);
              if(graphName!=="")
              {
                await deleteGraph(graphName).then((response)=>{
                  //console.log(response);
                });
              }
              //console.log(this.props.datasetDetailsCache);
              this.setState(
                {
                  datasetDetails: [...this.props.datasetDetailsCache]
                }
              );
              if(this.state.searchString!=="")
              {
                this.searchHandler();
              }

            });
        }
        else {

          let parameter = {
            datasetID: dataSetID
          };
          getDataset(parameter).then((response)=>{
            if(response.status===404)
            {
              this.setState({
                  isloading: false,
                  //loadFailed: true
              },async ()=>{
                await this.props.removeDatasetDetailsCache(dataSetID);
                if(graphName!=="")
                {
                  await deleteGraph(graphName).then((response)=>{
                    //console.log(response);
                  });
                }

                //console.log(this.props.datasetDetailsCache);
                this.setState(
                  {
                    datasetDetails: [...this.props.datasetDetailsCache]
                  }
                );
                if(this.state.searchString!=="")
                {
                  this.searchHandler();
                }

              });
            }
            else {
              this.setState({
                  isloading: false,
                  //loadFailed: true
              });
            }
          });

        }
      });
      });
    }

    addDataSetComponent = async (dataSetDetails) => {

      await this.props.addDatasetDetailsCache(dataSetDetails);
      //console.log(this.props.datasetDetailsCache);
      this.setState(
        {
          datasetDetails: [...this.props.datasetDetailsCache]
        }
      );
      if(this.state.searchString!=="")
      {
        this.searchHandler();
      }
    }

    updateDataSetComponent = async (dataSetDetails) => {

      await this.props.updateDatasetDetailsCache(dataSetDetails);
      //console.log(this.props.datasetDetailsCache);
      this.setState(
        {
          datasetDetails: [...this.props.datasetDetailsCache]
        }
      );
      if(this.state.searchString!=="")
      {
        this.searchHandler();
      }
    }

    changeView = (event) => {
        this.setState({
            selectedView: event.target.innerText
        }
        );
    }

    searchHandler = (event) => {
        let searchString="";
        let regExpr = /[^a-zA-Z0-9-_ ]/g;
        if(typeof event==="undefined")
        {
          searchString = this.state.searchString.replace(regExpr, "");
        }
        else {
          searchString = event.target.value.replace(regExpr, "");
        }

        if (searchString === "") {
            this.setState(
                {
                    searchString: searchString,
                    datasetDetails: [...this.props.datasetDetailsCache]
                }
            );
        }
        else {
            let tempList = [];
            if (this.props.datasetDetailsCache.length > 0) {
                tempList = this.props.datasetDetailsCache.filter((dataset) => {
                    let tempDataSetName = (dataset.datasetName).toUpperCase();
                    if (tempDataSetName.search(searchString.toUpperCase()) >= 0) {
                        return true;
                    }
                    return false;
                })
            }
            if (typeof tempList[0] == 'undefined') {
                tempList = [];
            }
            this.setState(
                {
                    searchString: searchString,
                    datasetDetails: tempList
                }
            );
        }

    }

openReConfigPopup = (datasetDetails) =>
{
  let hasExistingFile=false;
  if(datasetDetails.fileName!=="")
  {
    hasExistingFile=true;
  }
this.setState({
  connectDataSet : {
    datasetID : datasetDetails.datasetID,
    datasetName:datasetDetails.datasetName,
    datasetPLD:datasetDetails.datasetPLD,
    isSparqlEndPoint:datasetDetails.isSparqlEndPoint,
    sparqlEndPointURL:datasetDetails.sparqlEndPoint,
    knowledgeBaseID: datasetDetails.knowledgeBaseID,
    prevAssessmentDetails : [...datasetDetails.assessmentMetrics],
    datasetNameDisabled : true,
    datasetPLDDisabled : true,
    fileUploadedPath : (datasetDetails.fileName).substring(0, (datasetDetails.fileName).lastIndexOf("/") + 1),
    fileUploadedName : (datasetDetails.fileName).substring((datasetDetails.fileName).lastIndexOf("/") + 1, (datasetDetails.fileName).length),
    displayUseExistingFileCheckBox : hasExistingFile,
    useExistingFile : hasExistingFile,
    lastAssessmentRequestID: datasetDetails.lastAssessmentRequestID,
    graphName : (datasetDetails.fileName).substring((datasetDetails.fileName).lastIndexOf("/") + 1, (datasetDetails.fileName).lastIndexOf("."))
  }
}, ()=> {this.toggleConfigModel()});
}

    loading = () => <div className="animated fadeIn pt-1 text-center">Loading...</div>

    render() {
        let datasetList = null;
        if (this.state.datasetDetails.length > 0) {
            datasetList = (
                this.state.datasetDetails.map((dataset) => {
                  if(dataset.datasetID != 1){
                    let graphName = (dataset.fileName).substring((dataset.fileName).lastIndexOf("/") + 1, (dataset.fileName).lastIndexOf("."));
                    return <DataSetLarge key={dataset.datasetID} dataSetDetails={dataset} onRemoveClick={this.removeDataSetComponent.bind(this, dataset.datasetID, graphName)} onConfigClick={this.openReConfigPopup} whenUpdated={this.updateDataSetComponent}/>
                  }
                })
            );
        }
        return (

            <div className="animated fadeIn">
                {this.state.isloading ? <LoadingSpinner /> : null}
                {this.state.loadFailed ? <LoadingFailed clickHandler={this.toggleAlertModel} /> : null}
                {this.state.showTripleStoreError?<TripleStoreUploadError clickHandler={this.toggleTripleStoreErrorModel} graphName={this.state.graphName} />:null}
                <InputGroup>
                    <Input placeholder="Search" onChange={this.searchHandler} value={this.state.searchString} />
                </InputGroup>
                <br />



                <Row >
                    <Col>
                        <Dropdown isOpen={this.state.dropdownOpen} toggle={this.toggleDropDown} className="float-right">
                            <DropdownToggle caret>
                                {this.state.selectedView}
                            </DropdownToggle>
                            <DropdownMenu>
                                {this.state.selectedView === "Large Title View" ? <DropdownItem className="active" onClick={this.changeView}>Large Title View</DropdownItem> : <DropdownItem onClick={this.changeView}>Large Title View</DropdownItem>}
                                {this.state.selectedView === "Medium Title View" ? <DropdownItem className="active" onClick={this.changeView}>Medium Title View</DropdownItem> : <DropdownItem onClick={this.changeView}>Medium Title View</DropdownItem>}
                                {this.state.selectedView === "Small Title View" ? <DropdownItem className="active" onClick={this.changeView}>Small Title View</DropdownItem> : <DropdownItem onClick={this.changeView}>Small Title View</DropdownItem>}
                                {this.state.selectedView === "List" ? <DropdownItem className="active" onClick={this.changeView}>List</DropdownItem> : <DropdownItem onClick={this.changeView}>List</DropdownItem>}
                            </DropdownMenu>
                        </Dropdown>
                    </Col>
                </Row>
                <br />
                {this.state.showConfigModal ? <DatasetConnectConfigModelWindow onClose={this.toggleConfigModel} onNext={this.toggleBetweenConfigAndOptionModel} currentState={this.state.connectDataSet}/> : null}
                {this.state.showOptionModel ? <DatasetConnectOptionsModelWindow onBack={this.toggleBetweenConfigAndOptionModel} onSubmit={this.toggleOptionModel} currentState={this.state.connectDataSet} whenUpdateExisting={this.updateDataSetComponent} whenAddNew={this.addDataSetComponent}/> : null}
                <Row>
                    <ConnectDataSet tileClicked={this.toggleConfigModel} />
                    {datasetList}
                </Row>
            </div>
        );
    }
}

const mapStateToProps = (state) => {
    return (
        {
            datasetDetailsCache: state.datasetDetailsCache
        }
    );
}

const mapDispatchToProps = (dispatch) => {
    return ({
        updateDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'UPDATE_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); },
        addDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'ADD_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); },
        removeDatasetDetailsCache: (datasetID) => { return dispatch({ type: 'REMOVE_DATASET_DETAILS_CACHE', payLoad: {datasetID:datasetID} }); },
        createDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'CREATE_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); }
    }
    );
}
export default connect(mapStateToProps, mapDispatchToProps)(Dashboard);
