//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component } from 'react';
import {
  FormGroup,
  Form,
  Label,
  Input,
  Row,
  Col,
  Card,
  ListGroup,
  CardBody,
  CardText
} from 'reactstrap';
import MetricFilter from './MetricFilter';
import ResourceList from './ResourceList';
import ProblematicThingList from './ProblematicThingList';
import ResourceViewDetails from './DetailsModels/ResourceViewDetails';
import ProblematicThingViewDetails from './DetailsModels/ProblematicThingViewDetails';
import viewMapping from '../../../config/metricViewMapping';

class AnalysisWidget extends Component {
  constructor(props) {
    //console.log("constructor");

    super(props);
    this.state = {
      resourceView: false,
      problematicThingView: false,
      viewChanged: false,
      analysisMetrics: [],
      currentMetricFilter: [],
      resourceWiseList: [],
      problematicWiseList: [],
      showDetailsModel: false,
      showDetails: null,
      modelLinkNavigation: []
    }
    //console.log(this.props);
  }

  componentDidUpdate() {
    //console.log("componentDidUpdate");
  }

  componentDidMount() {
    //console.log("componentDidMount");
  }

  initialize = () => {
    //console.log("initialize");

    if (this.state.viewChanged) {
      //console.log("Changing Filter");
      let knowledgeBaseCache = JSON.parse(sessionStorage.getItem(this.props.knowledgeBaseID));
      if (knowledgeBaseCache !== null) {
        //console.log("Not Null");
        let tempMetricList = [...new Set(knowledgeBaseCache.map((knowledgeBase) => (knowledgeBase.FailedMertic)))];
        let viewType = "";
        if (this.state.problematicThingView) {
          viewType = "ProblematicThing";
        }
        else if (this.state.resourceView) {
          viewType = "Resource";
        }
        if (viewType !== "") {
          //tempMetricList = tempMetricList.filter((metric)=>(viewMapping.viewMapping[metric]===viewType));
          tempMetricList = tempMetricList.filter((metric) => {
            if (viewMapping.viewMapping[metric].indexOf(viewType) >= 0) {
              return metric;
            }
          });

          //console.log(tempMetricList);
          this.setState({
            analysisMetrics: tempMetricList,
            currentMetricFilter: [],
            viewChanged: false
          });
        }
      }
    }
  }

  changeView = (event) => {
    //console.log(event);
    if (event.target.value === "resource") {
      this.setState({
        resourceView: true,
        problematicThingView: false,
        viewChanged: true,
        analysisMetrics: [],
        currentMetricFilter: []
      }, () => {
        this.generateResourceWiseList();
      });
    }
    else if (event.target.value === "problematicThing") {
      this.setState({
        resourceView: false,
        problematicThingView: true,
        viewChanged: true,
        analysisMetrics: [],
        currentMetricFilter: []
      }, () => {
        this.generateProblematicThingWiseList();
      });
    }
  }

  generateResourceWiseList = () => {
    //console.log("generateResourceWiseList");
    let knowledgeBaseCache = JSON.parse(sessionStorage.getItem(this.props.knowledgeBaseID));
    let resourceDictionary = {};
    //knowledgeBaseCache.forEach((knowledgeBase)=>{
    for (const knowledgeBase of knowledgeBaseCache) {
      if (this.state.currentMetricFilter.indexOf(knowledgeBase.FailedMertic) >= 0) {
        let resource = knowledgeBase.Subject;
        if (resourceDictionary[resource] === undefined) {
          resourceDictionary[resource] = { exception: [], resourceReplica: [], disjointClass: [] };
        }

        //resourceDictionary[resource].exception = [...resourceDictionary[resource].exception, ...knowledgeBase.resourceRelatedExceptions];
        resourceDictionary[resource].exception = [...resourceDictionary[resource].exception, ...knowledgeBase.SubjectRelatedException];

        if (knowledgeBase.Exception === "ResourceReplica") {
          //if(resourceDictionary[resource].resourceReplica===null)
          //{
          //  resourceDictionary[resource].resourceReplica = [];
          //  resourceDictionary[resource].resourceReplica.push(knowledgeBase.Object);
          //}
          //else
          //{
          if (resourceDictionary[resource].resourceReplica.indexOf(knowledgeBase.Object) < 0) {
            resourceDictionary[resource].resourceReplica.push(knowledgeBase.Object);
          }
          //}
        }

        if (knowledgeBase.Exception === "MultiTypedResourceWithDisjointedClasses") {
          //if(resourceDictionary[resource].disjointClass===null)
          //{
          //  resourceDictionary[resource].disjointClass = [];
          //  resourceDictionary[resource].disjointClass.push(knowledgeBase.Object);
          //}
          //else
          //{
          if (resourceDictionary[resource].disjointClass.indexOf(knowledgeBase.Object) < 0) {
            resourceDictionary[resource].disjointClass.push(knowledgeBase.Object);
          }
          //}
        }

      }
    }
    //});

    let resourceArray = [];
    for (const [key, value] of Object.entries(resourceDictionary)) {
      //console.log(key, value);
      value.exception = [...new Set(value.exception.map((exception) => (exception)))];
      resourceArray.push({ [key]: value })
    }
    resourceArray.sort((resourcePrev, resourceNext) => {
      if (resourcePrev[Object.keys(resourcePrev)].resourceReplica !== null & resourceNext[Object.keys(resourceNext)].resourceReplica !== null & resourcePrev[Object.keys(resourcePrev)].resourceReplica.length > resourceNext[Object.keys(resourceNext)].resourceReplica.length) return -1;
      if (resourcePrev[Object.keys(resourcePrev)].resourceReplica !== null & resourceNext[Object.keys(resourceNext)].resourceReplica !== null & resourcePrev[Object.keys(resourcePrev)].resourceReplica.length < resourceNext[Object.keys(resourceNext)].resourceReplica.length) return 1;
      if (resourcePrev[Object.keys(resourcePrev)].disjointClass !== null & resourceNext[Object.keys(resourceNext)].disjointClass !== null & resourcePrev[Object.keys(resourcePrev)].disjointClass.length > resourceNext[Object.keys(resourceNext)].disjointClass.length) return -1;
      if (resourcePrev[Object.keys(resourcePrev)].disjointClass !== null & resourceNext[Object.keys(resourceNext)].disjointClass !== null & resourcePrev[Object.keys(resourcePrev)].disjointClass.length < resourceNext[Object.keys(resourceNext)].disjointClass.length) return 1;
      if (resourcePrev[Object.keys(resourcePrev)].exception.length > resourceNext[Object.keys(resourceNext)].exception.length) return -1;
      if (resourcePrev[Object.keys(resourcePrev)].exception.length < resourceNext[Object.keys(resourceNext)].exception.length) return 1;
    });
    //console.log(resourceArray);
    this.setState({
      resourceWiseList: resourceArray
    });
  }


  generateProblematicThingWiseList = () => {
    //console.log("generateProblematicThingWiseList");
    let knowledgeBaseCache = JSON.parse(sessionStorage.getItem(this.props.knowledgeBaseID));
    let problematicThingDictionary = {};
    //knowledgeBaseCache.forEach((knowledgeBase)=>{
    for (const knowledgeBase of knowledgeBaseCache) {
      if (this.state.currentMetricFilter.indexOf(knowledgeBase.FailedMertic) >= 0) {
        for (const problematicThing of knowledgeBase.ProblematicThing) {
          let problem = "";
          if (problematicThing === "Subject") {
            problem = "Subject-> " + knowledgeBase.Subject;
          }
          else if (problematicThing === "Predicate") {
            problem = "Predicate-> " + knowledgeBase.Predicate;
          }
          else if (problematicThing === "Object") {
            problem = "Object-> " + knowledgeBase.Object;
          }

          if (problem !== "") {
            if (problematicThingDictionary[problem] === undefined) {
              problematicThingDictionary[problem] = { exception: [], effectedResources: [], linkedProblematicThings: [] };
            }
            //problematicThingDictionary[problem].exception = [...problematicThingDictionary[problem].exception, ...knowledgeBase.problematicThingRelatedExceptions, knowledgeBase.Exception];
            if (problematicThing === "Subject") {
              problematicThingDictionary[problem].exception = [...problematicThingDictionary[problem].exception, ...knowledgeBase.SubjectRelatedException];
            }
            else if (problematicThing === "Predicate") {
              problematicThingDictionary[problem].exception = [...problematicThingDictionary[problem].exception, ...knowledgeBase.PredicateRelatedException];
            }
            else if (problematicThing === "Object") {
              problematicThingDictionary[problem].exception = [...problematicThingDictionary[problem].exception, ...knowledgeBase.ObjectRelatedException];
            }

            problematicThingDictionary[problem].effectedResources = [...problematicThingDictionary[problem].effectedResources, knowledgeBase.Subject];

            for (const linkedProblematicThing of knowledgeBase.ProblematicThing) {
              if (linkedProblematicThing !== problematicThing) {
                problematicThingDictionary[problem].linkedProblematicThings = [...problematicThingDictionary[problem].linkedProblematicThings, linkedProblematicThing + "-> " + knowledgeBase[linkedProblematicThing]];
              }
            }
          }
        }
      }
    }
    //});
    let problematicThingArray = [];
    for (const [key, value] of Object.entries(problematicThingDictionary)) {
      //console.log(key, value);
      value.exception = [...new Set(value.exception.map((exception) => (exception)))];
      value.effectedResources = [...new Set(value.effectedResources.map((effectedResource) => (effectedResource)))];
      value.linkedProblematicThings = [...new Set(value.linkedProblematicThings.map((linkedProblematicThing) => (linkedProblematicThing)))];
      problematicThingArray.push({ [key]: value })
    }

    problematicThingArray.sort((problematicThingPrev, problematicThingNext) => {
      if (problematicThingPrev[Object.keys(problematicThingPrev)].effectedResources.length > problematicThingNext[Object.keys(problematicThingNext)].effectedResources.length) return -1;
      if (problematicThingPrev[Object.keys(problematicThingPrev)].effectedResources.length < problematicThingNext[Object.keys(problematicThingNext)].effectedResources.length) return 1;
      if (problematicThingPrev[Object.keys(problematicThingPrev)].linkedProblematicThings.length > problematicThingNext[Object.keys(problematicThingNext)].linkedProblematicThings.length) return -1;
      if (problematicThingPrev[Object.keys(problematicThingPrev)].linkedProblematicThings.length < problematicThingNext[Object.keys(problematicThingNext)].linkedProblematicThings.length) return 1;
      if (problematicThingPrev[Object.keys(problematicThingPrev)].exception.length > problematicThingNext[Object.keys(problematicThingNext)].exception.length) return -1;
      if (problematicThingPrev[Object.keys(problematicThingPrev)].exception.length < problematicThingNext[Object.keys(problematicThingNext)].exception.length) return 1;

    });
    //console.log(problematicThingArray);
    this.setState({
      problematicWiseList: problematicThingArray
    });
  }

  updateMetricFilter = (event) => {
    //console.log(event.target.value);
    //console.log(event.target.checked);
    if (event.target.checked) {
      let tempFilter = this.state.currentMetricFilter;
      tempFilter.push(event.target.value);
      this.setState({
        currentMetricFilter: tempFilter
      }, () => {
        if (this.state.resourceView) {
          this.generateResourceWiseList();
        }
        else if (this.state.problematicThingView) {
          this.generateProblematicThingWiseList();
        }
      });
    }
    else {
      let tempFilter = this.state.currentMetricFilter.filter((metric) => { return metric !== event.target.value });
      this.setState({
        currentMetricFilter: tempFilter
      }, () => {
        if (this.state.resourceView) {
          this.generateResourceWiseList();
        }
        else if (this.state.problematicThingView) {
          this.generateProblematicThingWiseList();
        }
      });
    }
  }

  OpenLinkedProblematicThingModel = (problematicThing) => {
    //console.log(problematicThing);
    let indexFound = -1;
    for (const [index, value] of this.state.problematicWiseList.entries()) {
      if (Object.keys(value)[0] === problematicThing) {
        indexFound = index;
        break;
      }
    }
    if (indexFound >= 0) {
      let modelLinkNavigation = [...this.state.modelLinkNavigation];
      modelLinkNavigation.push(indexFound);
      this.setState({
        showDetailsModel: false,
        showDetails: null,
        modelLinkNavigation: [...modelLinkNavigation]
      }, () => {
        this.showDetails(indexFound);
      });
    }
  }

  closeShowDetails = () => {
    this.setState({
      showDetailsModel: false,
      showDetails: null,
      modelLinkNavigation: []
    });
  }

  showDetails = (index) => {
    //console.log("Showing Details : " + index);

    if (this.state.resourceView) {
      //console.log(this.state.resourceWiseList[index]);
      //console.log(this.props);
      //Fetch Complete Triples from the datastore
      //Single tab, list all the triples and mark the triple with exception
      //If there is resourcereplica exception then in another tab, list all the resources which are replica
      this.setState({
        showDetailsModel: true,
        showDetails: this.state.resourceWiseList[index]
      });
    }
    else if (this.state.problematicThingView) {
      //console.log(this.state.problematicWiseList[index]);
      if (this.state.modelLinkNavigation.length > 0) {
        this.setState({
          showDetailsModel: true,
          showDetails: this.state.problematicWiseList[index]
        });
      }
      else {
        this.setState({
          showDetailsModel: true,
          showDetails: this.state.problematicWiseList[index],
          modelLinkNavigation: [index]
        });
      }
    }
  }

  render() {
    //console.log("Render");
    //console.log(this.state);
    this.initialize();

    return (
      <React.Fragment>
        <Row className="row justify-content-md-center">
          <Col xs="4">
          </Col>
          <Col className="align-self-center" xs="8">
            <Form className="form-inline justify-content-center">
              <FormGroup check inline >
                <Input className="form-check-input" type="radio" name="inlineRadioOptions" id="resource" value="resource" checked={this.state.resourceView} onChange={this.changeView} />
                <Label className="form-check-label" for="inlineRadio1">Resource</Label>
              </FormGroup>
              <FormGroup check inline>
                <Input className="form-check-input" type="radio" name="inlineRadioOptions" id="problematicThing" value="problematicThing" checked={this.state.problematicThingView} onChange={this.changeView} />
                <Label className="form-check-label" for="inlineRadio2">Problematic Thing</Label>
              </FormGroup>
            </Form>
          </Col>
        </Row>
        <Row>
          <Col xs="4">
            {!this.state.problematicThingView & !this.state.resourceView ?
              <Card className="w-100 text-center" style={{ height: '30rem' }}>
                <p className="text-center font-weight-bold">Metrics</p>
                <CardBody>
                  <CardText>Please select either Resource View or Problematic View option to list metric filters.</CardText>
                  <CardText><small className="text-muted">If the required metric is not listed, then try switching the view option.</small></CardText>
                </CardBody>
              </Card>
              :
              <Card className="w-100" style={{ overflow: 'scroll', height: '30rem' }}>
                <p className="text-center font-weight-bold">Metrics</p>
                <ListGroup id="metricList" role="tablist" className="flex-column align-items-start">
                  <MetricFilter metrics={this.state.analysisMetrics} updateFilter={this.updateMetricFilter} datasetID={this.props.datasetID} />
                </ListGroup>
              </Card>}
          </Col>
          <Col xs="8">
            <Card className="w-100" style={{ overflow: 'scroll', height: '30rem' }}>
              <ListGroup id="metricList" role="tablist" className="flex-column align-items-start">
                {this.state.resourceView ? <ResourceList resourceList={this.state.resourceWiseList} showDetails={this.showDetails} /> : null}
                {this.state.problematicThingView ? <ProblematicThingList problematicThingList={this.state.problematicWiseList} showDetails={this.showDetails} /> : null}
              </ListGroup>
            </Card>
          </Col>
        </Row>
        {this.state.showDetailsModel & this.state.resourceView ? <ResourceViewDetails resourceDetails={this.state.showDetails} resourceList={this.state.resourceWiseList} closeModel={this.closeShowDetails} knowledgeBaseID={this.props.knowledgeBaseID} datasetID={this.props.datasetID} /> : null}
        {this.state.showDetailsModel & this.state.problematicThingView ? <ProblematicThingViewDetails problematicThingDetails={this.state.showDetails} closeModel={this.closeShowDetails} knowledgeBaseID={this.props.knowledgeBaseID} datasetID={this.props.datasetID} linkNavigation={this.OpenLinkedProblematicThingModel} currentMetrics={this.state.currentMetricFilter}/> : null}
      </React.Fragment>
    );
  }
}

export default AnalysisWidget;
