//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component } from 'react';
import { Nav, NavItem, NavLink, TabContent, UncontrolledTooltip, Card, Table, TabPane, Modal, Col, Row, ModalBody, ModalHeader, ModalFooter, Button, ListGroupItem } from 'reactstrap';
import classnames from 'classnames';
import properties from '../../../../config/dashboardProperties';
import exceptionInfoIconDetails from '../../../../config/exceptionInfoIconDetails';
import { connect } from 'react-redux';
import { read } from '../../../../services/tripleStoreAPIs/readFromTripleStore';
import { fetchTriplesForResource } from '../../../../services/tripleStoreAPIs/sparql/fetchTriplesForResource';
import { fetchListOfSimilarResource } from '../../../../services/tripleStoreAPIs/sparql/fetchListOfSimilarResource';
import LoadingSpinner from '../../loading';
import LoadingFailed from '../../loadingFailed';

class ResourceViewDetails extends Component {
  constructor(props) {
    //console.log("ResourceViewDetails constructor");
    super(props);
    this.state = {
      resource: "",
      activeTab: "",
      navigationTabs: [],
      triples: [],
      isloading: false,
      dataGraph: "",
      graphType: "",
      similarResources: [],
      showSimilarResourcePopup: false,
      similarResourceTriple: [],
      similarResourceTripleException: []
    }
    //console.log(this.props);
  }

  toggleTab = (tabID) => {
    //console.log(typeof tabID);
    if (this.state.activeTab !== tabID) {
      this.setState({
        activeTab: tabID
      });
    }
  }

  initialize = async () => {
    let resource = Object.keys(this.props.resourceDetails)[0];
    let navigationTabs = [...new Set(this.props.resourceDetails[resource].exception.map((exception) => {
      if (exception === "ResourceReplica") {
        return "ResourceReplica";
      }
      return "Default";
    }))];

    let activeTab = "Default";
    if (!(navigationTabs.indexOf("Default") >= 0)) {
      navigationTabs.push("Default");
    }
    //navigationTabs.push("Similar Resources");
    navigationTabs.sort();

    let dataGraph = "";
    let graphType = "";
    if (this.props.datasetDetailsCache[0].isSparqlEndPoint) {
      dataGraph = this.props.datasetDetailsCache[0].sparqlEndPoint;
      graphType = "SERVICE";
    }
    else {
      let dataGraphName = this.props.datasetDetailsCache[0].fileName.replace(/^.*[\\/]/, '').split('.').slice(0, -1).join('.');
      dataGraph = "http://" + properties.tripleStore.host + ":" + properties.tripleStore.port + "/" + properties.tripleStore.datastore + "/data/" + dataGraphName;
      graphType = "GRAPH";
    }
    //console.log("Going to fetch for : " +graphType+ ":"+dataGraph+":"+resource);
    let triples = await this.fetchDetailsAboutResource(graphType, dataGraph, resource);
    let [query, distinctProperties] = await this.prepareQueryToFindSimilarResources(triples);
    let similarResources = await this.findSimilarResources(graphType, dataGraph, query, resource);
    //console.log(query);
    //console.log(distinctProperties);
    //console.log(similarResources);
    let actualListOfSimilarResource = [];
    /*
    //What happens if there are 100 or more similar resources??
    for (const distinctResource of similarResources) {
      let triplesOfDistinctResource = await this.fetchDetailsAboutResource(graphType, dataGraph, distinctResource);
      let [queryOfDistinctResource, distinctPropertiesOfDistinctResource] = await this.prepareQueryToFindSimilarResources(triplesOfDistinctResource);
      let matching = await this.matchProperties(distinctProperties, distinctPropertiesOfDistinctResource);
      if (matching) {
        actualListOfSimilarResource.push(distinctResource);
      }
    }
    */
    return [resource, navigationTabs, activeTab, dataGraph, graphType, triples, actualListOfSimilarResource]
  }

  matchProperties = (distinctProperties, distinctPropertiesOfDistinctResource) => {
    let similar = false;
    return new Promise((resolve) => {

      if (distinctProperties.length === distinctPropertiesOfDistinctResource.length) {
        distinctProperties.sort();
        distinctPropertiesOfDistinctResource.sort();
        for (const [index, property] of distinctProperties.entries()) {
          if (property !== distinctPropertiesOfDistinctResource[index]) {
            similar = false;
            resolve(similar);
          }
        }
        similar = true;
        resolve(similar);
      }
      else {
        resolve(similar);
      }
    });
  }

  findSimilarResources = (graphType, dataGraph, query, resource) => {
    let similarResourcesFound = [];
    return new Promise((resolve) => {

      read(fetchListOfSimilarResource(graphType, dataGraph, query, resource)).then((response) => {

        //console.log(response);
        if (response) {
          if (response.results.bindings.length > 0) {
            for (const result of response.results.bindings) {
              similarResourcesFound.push("<" + result.Resource.value + ">");
            }
            //console.log("Returning Similar Resources");
            //console.log(similarResourcesFound);
            resolve(similarResourcesFound);
          }
          else {
            resolve(similarResourcesFound);
          }
        }
        else {
          resolve(similarResourcesFound);
        }
      });
    });
  }

  prepareQueryToFindSimilarResources = (triples) => {
    let query = "";
    let addedProperties = [];
    return new Promise((resolve) => {
      for (const [index, triple] of triples.entries()) {
        if (addedProperties.indexOf(triple.Predicate) < 0) {
          if (triple.Predicate !== "<http://www.w3.org/1999/02/22-rdf-syntax-ns#type>") {
            addedProperties.push(triple.Predicate);
          }
          if (index === 0) {
            query = "?Resource " + triple.Predicate + " ?Object_" + index;
          }
          else {
            query += triple.Predicate + " ?Object_" + index;
          }
          if ((index + 1) !== triples.length) {
            query += "; "
          }
        }
      }
      resolve([query, addedProperties]);
    });
  }

  fetchDetailsAboutResource = (graphType, dataGraph, resource) => {
    let triples = [];

    return new Promise((resolve) => {
      read(fetchTriplesForResource(graphType, dataGraph, resource)).then((response) => {
        //console.log(response);
        if (response) {
          if (response.results.bindings.length > 0) {
            let responseArray = 0;
            for (const triple of response.results.bindings) {
              //console.log("Respone Array : " + responseArray);
              let object = "";
              if (triple.Object.type === "uri") {
                object += "<" + triple.Object.value + ">";
              }
              else {
                if (triple.Object["xml:lang"] !== undefined) {
                  object += "\"\"\"" + triple.Object.value + "\"\"\"@" + triple.Object["xml:lang"];
                }
                else if (triple.Object["datatype"] !== undefined) {
                  let value = triple.Object.value.replace(/("|')/g, '\\$1');
                  object += "\"\"\"" + value + "\"\"\"" + "^^<" + triple.Object["datatype"] + ">";
                }
                else {
                  object += "\"\"\"" + triple.Object.value + "\"\"\"";
                }
              }
              triples.push({
                Subject: "<" + triple.Resource.value + ">",
                Predicate: "<" + triple.Predicate.value + ">",
                Object: object,
                Exception: []
              });
              responseArray += 1;
            }
            //let knowledgeBaseCache = JSON.parse(sessionStorage.getItem(this.props.knowledgeBaseID));
            //let knowledgeBaseCache = this.props.knowledgeBaseCache;
            let tripleArray = 0;
            for (const triple of triples) {
              //console.log("Inside Triple Array : "+tripleArray);
              for (const knowledgeBase of this.props.knowledgeBaseCache) {
                if (knowledgeBase.Subject === triple.Subject & knowledgeBase.Predicate === triple.Predicate & knowledgeBase.Object === triple.Object) {
                  triple.Exception.push(knowledgeBase.Exception);
                }
                if(knowledgeBase.Object === triple.Subject & knowledgeBase.Exception==="ResourceReplica")
                {
                  triple.Exception.push(knowledgeBase.Exception);
                }
              }
              tripleArray += 1;
            }
            //console.log("Returning Triples");
            //console.log(triples);
            resolve(triples);
          }
          else {
            resolve(triples);
          }
        }
        else {
          resolve(triples);
        }
      });
    });
  }

  componentDidMount() {

    this.setState({
      isloading: true
    }, () => {
      this.initialize().then((results) => {
        this.setState({
          resource: results[0],
          navigationTabs: results[1],
          activeTab: results[2],
          dataGraph: results[3],
          graphType: results[4],
          triples: results[5],
          similarResources: results[6],
          isloading: false
        });
      }
      );
    });
  }

  closeSimilarResourcePopup = () => {
    this.setState({
      showSimilarResourcePopup: false,
      similarResourceTriple: [],
      similarResourceTripleException: []
    });
  }

  showSimilarResource = async (resource) => {
    //console.log(resource);
    let triples = await this.fetchDetailsAboutResource(this.state.graphType, this.state.dataGraph, resource);
    let tempExceptionsOfSimilarResource = [];
    for (const triple of triples) {
      tempExceptionsOfSimilarResource=[...tempExceptionsOfSimilarResource,...triple.Exception]
    }
      let exceptionsOfSimilarResource=[...new Set(tempExceptionsOfSimilarResource.map((exception) => exception))];
    this.setState({
      showSimilarResourcePopup: true,
      similarResourceTriple: triples,
      similarResourceTripleException: exceptionsOfSimilarResource
    });
  }

  renderDetails = (tabID) => {
    switch (tabID) {
      case "Default":
        return (<React.Fragment>
          <p style={{ fontSize: "14px" }} className="text-center">Exception(s) Related to this Resource : {this.props.resourceDetails[this.state.resource].exception.map((exception, indexException) => { return <React.Fragment key={indexException}><i>{exception} </i><i className="fa fa-info-circle" id={exception + "_toolTipIcon"}>  </i><UncontrolledTooltip placement="right" target={exception + "_toolTipIcon"}>{exceptionInfoIconDetails.infoIcon[exception]}</UncontrolledTooltip><span>&nbsp;</span></React.Fragment> })}</p>
          <font size="2"  >
            <Table hover bordered><thead><tr><th>Subject</th><th>Predicate</th><th>Object</th><th>Exception(s)</th></tr></thead><tbody>
              {this.state.triples.map((triple, indexInside) => {
                if (triple.Exception.length > 0) {
                  return <tr key={indexInside} style={{ 'backgroundColor': 'rgba(255,255,102,0.5)' }} ><td>{triple.Subject}</td><td>{triple.Predicate}</td><td>{triple.Object}</td><td>{triple.Exception.join(', ')}</td></tr>
                }
                return <tr key={indexInside}><td>{triple.Subject}</td><td>{triple.Predicate}</td><td>{triple.Object}</td><td></td></tr>
              })}
            </tbody></Table></font>
        </React.Fragment>);

      case "Similar Resources":
        return (<React.Fragment>
          {this.state.similarResources.length > 0 ?
            <Table hover bordered striped><thead><tr><th>Similar Resources <small>(Click on the link to view)</small></th></tr></thead><tbody>
              {this.state.similarResources.map((resource, indexInside) => {
                return <tr key={indexInside}><td onClick={() => this.showSimilarResource(resource)}>{resource}</td></tr>
              })}
            </tbody></Table>
            :
            <p style={{ fontSize: "14px" }} className="text-center">No similar resource(s) found</p>
          }
        </React.Fragment>);

      case "ResourceReplica":
        return (<React.Fragment>
          {this.props.resourceDetails[this.state.resource].resourceReplica.length>0?
            this.props.resourceDetails[this.state.resource].resourceReplica.map((resourceReplica, indexInside) => {
              return <ListGroupItem key={indexInside} className="border-0">{resourceReplica}</ListGroupItem>
            })
            :
            <p style={{ fontSize: "14px" }} className="text-center">Please select ExtensionalConcisenessMetric from the filter list.</p>
          }
        </React.Fragment>);
    }
  }

  render() {
    //console.log(this.state);
    let navigationTabs = null;
    let displayPane = null;
    navigationTabs = (this.state.navigationTabs.map((tabID, index) => {
      return <NavItem key={index}><NavLink className={classnames({ active: this.state.activeTab === tabID })} onClick={() => { this.toggleTab(tabID); }}>{tabID}</NavLink></NavItem>
    }));

    displayPane = (this.state.navigationTabs.map((tabID, index) => {
      return <TabPane tabId={tabID} style={{ overflow: 'scroll', height: '30rem' }} key={index}><Row><Col>
        {
          this.renderDetails(tabID)
        }
      </Col></Row></TabPane>
    }));
    let similarResourcePopup = null;
    similarResourcePopup = (
      <React.Fragment>
        <Modal isOpen={true} className="modal-dialog-centered mw-100 w-75" >
          <ModalHeader style={{ margin: 'auto' }}>Similar Resource Details</ModalHeader>
          <ModalBody>
            <Card style={{ overflow: 'scroll', height: '30rem' }} ><Row><Col>
              <p style={{ fontSize: "14px" }} className="text-center">Exception(s) Related to this Resource : {this.state.similarResourceTripleException.map((exception, indexException) => { return <React.Fragment key={indexException}><i>{exception} </i><i className="fa fa-info-circle" id={exception + "_toolTipIcon_similar"}>  </i><UncontrolledTooltip placement="right" target={exception + "_toolTipIcon_similar"}>{exceptionInfoIconDetails.infoIcon[exception]}</UncontrolledTooltip><span>&nbsp;</span></React.Fragment> })}</p>
              <font size="2"  >
                <Table hover bordered><thead><tr><th>Subject</th><th>Predicate</th><th>Object</th><th>Exception(s)</th></tr></thead><tbody>
                  {this.state.similarResourceTriple.map((triple, indexInside) => {
                    if (triple.Exception.length > 0) {
                      return <tr key={indexInside} style={{ 'backgroundColor': 'rgba(255,255,102,0.5)' }} ><td>{triple.Subject}</td><td>{triple.Predicate}</td><td>{triple.Object}</td><td>{triple.Exception.join(', ')}</td></tr>
                    }
                    return <tr key={indexInside}><td>{triple.Subject}</td><td>{triple.Predicate}</td><td>{triple.Object}</td><td></td></tr>
                  })}
                </tbody></Table>
              </font>
            </Col></Row></Card>
          </ModalBody>
          <ModalFooter>
            <Button color="secondary" onClick={this.closeSimilarResourcePopup}>Ok</Button>
          </ModalFooter>
        </Modal>
      </React.Fragment>
    );

    return (
      <React.Fragment>
        <Modal isOpen={true} className="modal-dialog-centered mw-100 w-100" >
          <ModalHeader style={{ margin: 'auto' }}>Resource Details</ModalHeader>
          <ModalBody>
            <Nav tabs>
              {navigationTabs}
            </Nav>
            <TabContent activeTab={this.state.activeTab}>
              {displayPane}
            </TabContent>
          </ModalBody>
          <ModalFooter>
            <Button color="secondary" onClick={this.props.closeModel}>Ok</Button>
          </ModalFooter>
          {this.state.isloading ? <LoadingSpinner /> : null}
          {this.state.showSimilarResourcePopup ? similarResourcePopup : null}
        </Modal>
      </React.Fragment>
    );
  }
}


const mapStateToProps = (state, ownProps) => {
  let datasetID = Number(ownProps.datasetID);
  let datasetDetails = state.datasetDetailsCache.filter((dataset) => { if (dataset.datasetID === datasetID) { return dataset; } return null; });
  return (
    {
      datasetDetailsCache: datasetDetails
    }
  );
}

export default connect(mapStateToProps)(ResourceViewDetails);
