//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component } from 'react';
import {
  Button, Progress
} from 'reactstrap';
import { connect } from 'react-redux';
import LoadingSpinner from '../loading';
import LoadingFailed from '../loadingFailed';
import { getKnowledgeBaseStatus, generateKnowledgeBase, getKnowledgeBase } from '../../../services/datasetConfigDetails/datasetConfigDetails';
import { updateDataset, getStatus } from '../../../services/datasetConfigDetails/datasetConfigDetails';
import { read } from '../../../services/tripleStoreAPIs/readFromTripleStore';
import { statusUpdate } from '../../../services/tripleStoreAPIs/statusUpdate';
import { checkDatasetExists } from '../../../services/tripleStoreAPIs/sparql/checkDatasetExists';
import properties from '../../../config/dashboardProperties';
import ProblemsModel from '../problemsModel'
import AnalysisWidget from './AnalysisWidget'
import metricExceptionMapping from '../../../config/metricKnowledgeBaseMapping';
import axios from 'axios';
import queryString from 'query-string';

/*-------------config-------------*/
const headers = { 'Accept': 'application/sparql-results+json', 'Content-Type': 'application/x-www-form-urlencoded' };
/*-----------------------------------*/


class PerformAnalysis extends Component {

  constructor(props) {
    super(props);
    this.state = {
      hasKnowledgeBase: false,
      knowledgeBaseGenerationInProgress: false,
      isloading: false,
      loadFailed: false,
      problemsNoticed: [],
      knowledgeBaseCache:null,
      percentage:0
    }
  }

  componentDidMount() {
    this.initialize();
  }

  recall = 0;

  performAnalysisStatusUpdate = async () => {
    this.recall = setInterval(async () => {
      //console.log("Call Status Update");
      getKnowledgeBaseStatus(this.props.datasetDetailsCache[0].knowledgeBaseID).then((response) => {
        //console.log("Got Status");
        if (response) {
          //console.log(response);
          if (response.status === 200) {
            if (response.data.status !== "INPROGRESS"  ) {
              clearInterval(this.recall);
              this.initialize();
            }
            else {
              this.setState({
                percentage: Number.parseFloat(response.data.percentage * 100).toFixed(2)
              });
            }

          }
          else {
            //console.log("ERROR");
            clearInterval(this.recall);
            this.initialize();
          }
        }
        else {
          //console.log("ERROR");
          clearInterval(this.recall);
          this.initialize();
        }

      });


    }, 5000);
  }

  componentWillUnmount() {
    //console.log("Component Will  Unmount");
    //console.log(this.recall);
    clearInterval(this.recall);
    //console.log(this.recall);
  }

  initialize = () => {
    //console.log("Initialize");
    //console.log(metricExceptionMapping.knowledgeBaseMapping);
    if (this.props.datasetDetailsCache[0].knowledgeBaseID !== "" & this.props.datasetDetailsCache[0].knowledgeBaseID !== undefined) {
      //console.log("knowledgeBaseID Present");
      //Check in session cache
      //let knowledgeBaseCache = JSON.parse(sessionStorage.getItem(this.props.datasetDetailsCache[0].knowledgeBaseID));
      if (this.state.knowledgeBaseCache === null) {
        getKnowledgeBaseStatus(this.props.datasetDetailsCache[0].knowledgeBaseID).then((response) => {
          //console.log("Got Status");
          if (response) {
            if (response.status === 200) {
              if (response.data.status === "COMPLETED") {
                //console.log("COMPLETED");
                clearInterval(this.recall);
                this.setState({
                  isloading: true
                }, async () => {
                  await getKnowledgeBase(this.props.datasetDetailsCache[0].knowledgeBaseID).then((responseKB) => {
                    //console.log("FETCHED");
                    let knowledgeBaseFetched = responseKB.data.knowledgeBase;
                    //sessionStorage.setItem(this.props.datasetDetailsCache[0].knowledgeBaseID, JSON.stringify(knowledgeBaseFetched));
                    this.setState({
                      isloading: false,
                      knowledgeBaseCache: knowledgeBaseFetched,
                      hasKnowledgeBase: true,
                      knowledgeBaseGenerationInProgress: false,
                      percentage:0
                    });
                    //console.log("STORED");
                  });
                });
              }
              else if (response.data.status === "FAILED") {
                clearInterval(this.recall);
                this.setState({
                  hasKnowledgeBase: false,
                  knowledgeBaseGenerationInProgress: false,
                  percentage:0
                });
              }
              else {
                //console.log(response);
                this.performAnalysisStatusUpdate();
                this.setState({
                  hasKnowledgeBase: false,
                  knowledgeBaseGenerationInProgress: true,
                  percentage: Number.parseFloat(response.data.percentage * 100).toFixed(2)
                });
              }
            }
            else {
              clearInterval(this.recall);
              this.setState({
                hasKnowledgeBase: false,
                knowledgeBaseGenerationInProgress: false,
                percentage:0
              });
            }
          }
          else {
            clearInterval(this.recall);
            this.setState({
              hasKnowledgeBase: false,
              knowledgeBaseGenerationInProgress: false,
              percentage:0
            });
          }
        });
      }
      else {
        //console.log("Inside ELSE");
        this.setState({
          hasKnowledgeBase: true,
          knowledgeBaseGenerationInProgress: false
        });
      }
    }
  }

  checkTripleStoreIsReachable = () => {
    let isReachable = false;

    return new Promise((resolve) => {
      this.setState({ isloading: true }, () => {
        statusUpdate().then((response) => {
          this.setState({ isloading: false }, () => {
            if (response) {
              if (response.status === 200) {
                isReachable = true;
                resolve(isReachable);
              }
              else {
                isReachable = false;
                resolve(isReachable);
              }
            }
            else {
              isReachable = false;
              resolve(isReachable);
            }
          });
        });
      });
    }
    );
  }

  checkDataGraphExistsInTripleStore = () => {
    let exists = false;
    let hasMultiple = false;
    let dataGraph = "";

    return new Promise((resolve) => {
      let dataGraphName = this.props.datasetDetailsCache[0].fileName.replace(/^.*[\\/]/, '').split('.').slice(0, -1).join('.');
      dataGraph = "http://" + properties.tripleStore.host + ":" + properties.tripleStore.port + "/" + properties.tripleStore.datastore + "/data/" + dataGraphName;
      this.setState({
        isloading: true,
        //loadFailed: false
      }, () => {
        read(checkDatasetExists(dataGraph)).then((response) => {
          this.setState({
            isloading: false
          }, () => {
            //console.log(response);
            if (response) {
              if (response.results.bindings.length === 1) {
                exists = true;
                resolve([exists, hasMultiple, dataGraph]);
              }
              else if (response.results.bindings.length > 1) {
                exists = true;
                hasMultiple = true;
                resolve([exists, hasMultiple, dataGraph]);
              }
              else {
                exists = false;
                resolve([exists, hasMultiple, dataGraph]);
              }
            }
            else {
              resolve([exists, hasMultiple, dataGraph]);
            }
          });
        });
      });
    });
  }

  checkSparqlEndPointIsReachable = () => {
    let isReachable = false;
    let query = "SELECT DISTINCT  (COUNT(?s) AS ?count) WHERE   {   { ?s  ?p  ?o }     UNION       { GRAPH ?g           { ?s  ?p  ?o }       }   }";
    return new Promise((resolve) => {
      //isReachable=true;
      this.setState({
        isloading: true,
        //loadFailed: false
      }, () => {

        axios.post(this.props.datasetDetailsCache[0].sparqlEndPoint, queryString.stringify({ query: query }), headers)
          .then((response) => {
            this.setState({ isloading: false }, () => {
              //console.log(response);
              if (response.status === 200) {
                isReachable = true;
                resolve(isReachable);
              }
              else {
                isReachable = false;
                resolve(isReachable);
              }
            });
          })
          .catch((err) => {
            this.setState({ isloading: false }, () => {
              isReachable = false;
              resolve(isReachable);
            });
          })
      }
      );
    });
  }

  checkWrapperServiceIsReachable = () => {
    let isReachable = false;

    return new Promise((resolve) => {
      this.setState({ isloading: true }, () => {
        getStatus().then((response) => {
          this.setState({ isloading: false }, () => {
            //console.log(response);
            if (response.status === 200) {
              if (response.data.status === "OK") {
                isReachable = true;
                resolve(isReachable);
              }
              else {
                isReachable = false;
                resolve(isReachable);
              }
            }
            else {
              isReachable = false;
              resolve(isReachable);
            }
          });
        });
      });
    }
    );
  }

  checkKnowledgeBaseDBIsReachable = () => {
    let isReachable = false;

    return new Promise((resolve, reject) => {
      isReachable = true;
      resolve(isReachable);
    }
    );
  }


  toggleAlertModel = () => {
    this.setState({
      loadFailed: false
    });
  }

  triggerKBGeneration = async () => {
    let problemsNoticedBeforeTriggering = [];
    let qualityGraph = [...new Set(this.props.datasetDetailsCache[0].lastAssessmentMetrics.map((metric) => { if (metric.ProblemGraph !== undefined) { return metric.ProblemGraph.value } }))].filter((graph) => { return graph !== undefined });
    let metrics = this.props.datasetDetailsCache[0].lastAssessmentMetrics.filter((metric) => { return metric.ProblemGraph !== undefined }).map((metric) => {

      let valueToReturn = {
        metric: metric.Metric.value.split("#")[1],
        observationURI: metric.ObservationURI.value
      };
      return valueToReturn;
    });
    let dataGraphExists = false;
    let hasMultipleDataGraph = false;
    let dataGraph = "";
    let wrapperServiceAvailable = false;
    let knowledgeBaseDBAvailable = false;
    let sparqlEndPointIsReachable = false;
    let isSparqlEndPoint = false;
    let tripleStoreIsReachable = false;

    wrapperServiceAvailable = await this.checkWrapperServiceIsReachable();
    knowledgeBaseDBAvailable = await this.checkKnowledgeBaseDBIsReachable();

    if (!this.props.datasetDetailsCache[0].isSparqlEndPoint) {
      //console.log("Not Sparql");
      tripleStoreIsReachable = await this.checkTripleStoreIsReachable();
      if (tripleStoreIsReachable) {
        [dataGraphExists, hasMultipleDataGraph, dataGraph] = await this.checkDataGraphExistsInTripleStore();
      }
      //console.log(dataGraphExists);
      //console.log(hasMultipleDataGraph);
      //console.log(dataGraph);
      isSparqlEndPoint = false;
    }
    else {
      sparqlEndPointIsReachable = await this.checkSparqlEndPointIsReachable();
      isSparqlEndPoint = true;
      dataGraph = this.props.datasetDetailsCache[0].sparqlEndPoint;
    }

    //Create Problem List
    if (!isSparqlEndPoint & !tripleStoreIsReachable) {
      problemsNoticedBeforeTriggering.push({ TripleStoreNotReachable: "Host:" + properties.tripleStore.host + ", Port:" + properties.tripleStore.port + ", Dataset:" + properties.tripleStore.datastore });
    }
    if (!isSparqlEndPoint & !dataGraphExists & tripleStoreIsReachable) {
      problemsNoticedBeforeTriggering.push({ DataSetGraphNotFound: dataGraph });
    }
    if (!isSparqlEndPoint & hasMultipleDataGraph & tripleStoreIsReachable) {
      problemsNoticedBeforeTriggering.push({ MultipleGraphsFound: dataGraph });
    }
    if (!wrapperServiceAvailable) {
      problemsNoticedBeforeTriggering.push({ WrapperServiceNotAvailable: "Host:" + properties.wrapperAPI.host + ", Port:" + properties.wrapperAPI.port });
    }
    if (!knowledgeBaseDBAvailable) {
      problemsNoticedBeforeTriggering.push({ KnowledgeBaseDBNotAvailable: "Host:" + properties.knowledgeBase.host + ", Port:" + properties.knowledgeBase.port });
    }
    if (isSparqlEndPoint & !sparqlEndPointIsReachable) {
      problemsNoticedBeforeTriggering.push({ SPARQLEndPointNotReachable: this.props.datasetDetailsCache[0].sparqlEndPoint });
    }

    if (problemsNoticedBeforeTriggering.length === 0) {
      this.setState({
        isloading: true,
        loadFailed: false
      }, () => {
        generateKnowledgeBase(metrics, qualityGraph[0], dataGraph, this.props.datasetDetailsCache[0].dataSetLastAssessed, isSparqlEndPoint, metricExceptionMapping.knowledgeBaseMapping).then((response) => {
          //console.log(response);
          if (response) {
            if (response.status === 201) {
              let detailsToUpdate = {
                datasetID: this.props.datasetDetailsCache[0].datasetID,
                knowledgeBaseID: response.data.knowledgeBaseID
              }

              this.props.updateDatasetDetailsCache(detailsToUpdate);
              this.performAnalysisStatusUpdate();
              this.setState({
                knowledgeBaseGenerationInProgress: true

              }, () => {
                updateDataset({ "knowledgeBaseID": response.data.knowledgeBaseID }, this.props.datasetDetailsCache[0].datasetID).then((response) => {
                  if (response.status === 200) {
                    this.setState({
                      isloading: false,
                      //loadFailed: false
                    });
                  }
                  else {
                    this.setState({
                      isloading: false,
                      //loadFailed: true
                    });
                  }
                });
              });
            }
          }
          else {
            this.setState({
              isloading: false,
              //loadFailed: true
            });
          }
        });
      });
    }
    else {
      //Has problems. Dont Continue. Show error popup.
      this.setState({
        problemsNoticed: problemsNoticedBeforeTriggering
      });
    }
  }

  problemModelClosed = () => {
    this.setState({
      problemsNoticed: []
    });
  }

  render() {
    //console.log(this.state);
    //console.log(this.props);

    //console.log("Render");
    let displayGenerateKBButton = true;
    if (this.props.datasetDetailsCache[0].dataSetLastAssessed === "Not Assessed Yet") {
      displayGenerateKBButton = false;
    }
    return (
      <React.Fragment>
        {displayGenerateKBButton & !this.state.hasKnowledgeBase ? <Button block color="secondary" aria-pressed="true" disabled={this.state.knowledgeBaseGenerationInProgress} onClick={this.triggerKBGeneration}>Perform Analysis</Button> : null}
        {displayGenerateKBButton & !this.state.hasKnowledgeBase & this.state.knowledgeBaseGenerationInProgress? <Progress className="progress-xs" color="info" style={{ 'height': '20px' }} value={this.state.percentage} ><small className="justify-content-center d-flex position-absolute w-100"><font size="2" color="black">Completed : {this.state.percentage}%</font></small></Progress> : null}
        {this.state.isloading ? <LoadingSpinner /> : null}
        {this.state.loadFailed ? <LoadingFailed clickHandler={this.toggleAlertModel} /> : null}
        {this.state.problemsNoticed.length > 0 ? <ProblemsModel clickHandler={this.problemModelClosed} triggerProblems={this.state.problemsNoticed} /> : null}
        {this.state.hasKnowledgeBase ? <AnalysisWidget knowledgeBaseID={this.props.datasetDetailsCache[0].knowledgeBaseID} datasetID={this.props.datasetDetailsCache[0].datasetID} knowledgeBaseCache={this.state.knowledgeBaseCache}/> : null}
      </React.Fragment>
    );
  }
}

const mapStateToProps = (state, ownProps) => {
  let datasetID = Number(ownProps.datasetID);
  let datasetDetails = state.datasetDetailsCache.filter((dataset) => { if (dataset.datasetID === datasetID) { return dataset; } return null; });
  return (
    {
      datasetDetailsCache: datasetDetails
    }
  );
}


const mapDispatchToProps = (dispatch) => {
  return ({
    updateDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'UPDATE_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); }
  }
  );
}
export default connect(mapStateToProps, mapDispatchToProps)(PerformAnalysis);
