//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component } from 'react';
import {
  Modal,
  ModalHeader,
  ModalBody,
  ModalFooter,
  Button,
  Row,
  Col,
  Card,
  CardBody,
  CardText
} from 'reactstrap';
import LoadingSpinner from '../loading';
import LoadingFailed from '../loadingFailed';
import properties from '../../../config/dashboardProperties';
import { connect } from 'react-redux';
import { read } from '../../../services/tripleStoreAPIs/readFromTripleStore';
import { fetchDistinctResourceCount } from '../../../services/tripleStoreAPIs/sparql/fetchDistinctResourceCount';


class SummaryReport extends Component {

  constructor(props) {
    super(props);
    this.state = {
      isloading: false,
      loadFailed: false,
      totalNumberOfResources: 0
    }
  }

  componentDidMount() {
    let dataGraph = "";
    let graphType = "";
    if (this.props.datasetDetailsCache[0].isSparqlEndPoint) {
      dataGraph = this.props.datasetDetailsCache[0].sparqlEndPoint;
      graphType = "SERVICE";
    }
    else {
      let dataGraphName = this.props.datasetDetailsCache[0].fileName.replace(/^.*[\\/]/, '').split('.').slice(0, -1).join('.');
      dataGraph = "http://" + properties.tripleStore.host + ":" + properties.tripleStore.port + "/" + properties.tripleStore.datastore + "/data/" + dataGraphName;
      graphType = "GRAPH";
    }

    this.setState({
      isloading: true,
      //loadFailed: false
    }, () => {
      read(fetchDistinctResourceCount(graphType,dataGraph)).then((response) => {
        this.setState({
          isloading: false
        }, () => {
          if (response) {
            if (response.results.bindings.length > 0) {
              this.setState({
                totalNumberOfResources: response.results.bindings[0].ResourceCount.value
              });
            }
            else {

            }
          }
          else {

          }
        });
      });
    });
  }


  render() {
    //console.log(this.state);
    let listOfMetricsToDisplay = null;
    listOfMetricsToDisplay = this.props.listOfMetrics.map((metric, index) => {
      let metricKey = Object.keys(metric);
      return <CardText key={index} className="text-center"><p className="text-center font-weight-normal" >{metricKey[0]} : {metric[metricKey[0]].length}</p></CardText>
    });

    let countOfTotalResource = null;
    countOfTotalResource =(
      <CardBody className="text-center">
        <p className="text-center font-weight-bold" ><u>Total number of Resource(s) in assessed dataset : {this.state.totalNumberOfResources}</u></p>
      </CardBody>
    );


    return (
      <Modal isOpen={true} className="modal-dialog-centered modal-md">
        <ModalHeader style={{ margin: 'auto' }}>Analysis Summary</ModalHeader>
        <ModalBody>
          <Row>
            <Col xs="12">
              <Row>
                <Card className="w-100 ">
                {this.state.totalNumberOfResources>0? countOfTotalResource :null}
                  <p className="text-center font-weight-bold" ><u>Number of Resource(s) impacted by each Metric(s)</u></p>
                  <CardBody className="text-center">
                    {listOfMetricsToDisplay}
                  </CardBody>
                </Card>
              </Row>
            </Col>
          </Row>
        </ModalBody>
        {this.state.isloading ? <LoadingSpinner /> : null}
        <ModalFooter>
          <Button color="secondary" onClick={this.props.onCloseButtonClick}>Close</Button>
        </ModalFooter>
      </Modal>

    );
  }

}



const mapStateToProps = (state, ownProps) => {
  let datasetID = Number(ownProps.datasetID);
  let datasetDetails = state.datasetDetailsCache.filter((dataset) => { if (dataset.datasetID === datasetID) { return dataset; } return null; });
  return (
    {
      datasetDetailsCache: datasetDetails
    }
  );
}

export default connect(mapStateToProps)(SummaryReport);
