//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component } from 'react';
import {
  Card,
  CardBody,
  CardHeader,
  Row,
  CardSubtitle,
  Col,
  Progress,
  Container
} from 'reactstrap';
import { connect } from 'react-redux';
import { Redirect } from 'react-router-dom';
import CircularProgressbar from 'react-circular-progressbar';
import 'react-circular-progressbar/dist/styles.css';
import { Link } from 'react-router-dom';
import ErrorReport from './ErrorReport';
import SummaryReport from './SummaryReport';
const noteReadyToPublish = "rgba(255,0,0,0.6)";
const readyToPublish = "rgba(34,139,34,0.8)";


class DimensionDetails extends Component {

  constructor(props) {
    super(props);
    this.state = {
      dimensionActual: 0,
      dimensionExpected: 0,
      dimension: this.props.match.params.dimension,
      metricValues: [],
      showErrorPopup: false,
      showErrorReportFor: null,
      showSummaryPopup: false,
      showSummaryReportFor: null
    }
  }

  componentDidMount() {
    if (this.props.datasetDetailsCache.length > 0) {
      this.calculateDimensionActualAndExpected(this.state.dimension);
    }
  }

  calculateDimensionActualAndExpected = (dimension) => {

    let tempArray = [];
    tempArray = this.props.datasetDetailsCache[0].assessmentMetrics.map((metric) => {
      return (this.props.datasetDetailsCache[0].lastAssessmentMetrics.map((assessed) => {
        if (metric.metric === assessed.Metric.value & metric.assess & assessed.Dimension.value.split("#")[1] === dimension) {
          let assessValue = 0;
          let problemStrucure = "";
          let qualityProblem = "";
          if (assessed.Value.datatype.split("#")[1] === "boolean") {
            if (assessed.Value.value === "true") {
              assessValue = 1.0;
            }
          }
          else {
            assessValue = Number(parseFloat(assessed.Value.value).toFixed(4));
          }

          if (assessed.QualityProblem !== undefined) {
            qualityProblem = assessed.QualityProblem.value;
          }

          if (assessed.ProblemStructure !== undefined) {
            problemStrucure = assessed.ProblemStructure.value;
          }

          return ({
            metric: assessed.Metric.value, observationURI: assessed.ObservationURI.value, actual: assessValue, target: metric.target, label: metric.label, comment: metric.comment, type: (assessed.Value.datatype).split("#")[1],
            qualityProblem: qualityProblem, problemStructure: problemStrucure
          });
        }
      }));
    });

    this.setState(
      {
        metricValues: [...tempArray.flat().filter(item => item)]
      }, () => {
        let tempActual = 0.0;
        let tempExpected = 0.0;
        this.state.metricValues.forEach((values) => {
          if (values.type === "boolean") {
            tempExpected += 1;
            if (Number(parseFloat(values.actual).toFixed(4)) === Number(parseFloat(values.target).toFixed(4))) {
              tempActual += 1;
            }
            else {
              tempActual += 0;
            }
          }
          else {
            tempActual += Number(parseFloat(values.actual).toFixed(4));
            tempExpected += Number(parseFloat(values.target).toFixed(4));
          }
        });

        if (this.state.metricValues.length > 0) {
          tempActual = tempActual / this.state.metricValues.length;
          tempExpected = tempExpected / this.state.metricValues.length;
        }

        this.setState({
          dimensionActual: tempActual,
          dimensionExpected: tempExpected,
        });
      }
    );
  }

  showErrorReport = (metricURI) => {
    this.setState(
      {
        showErrorPopup: true,
        showErrorReportFor: metricURI
      }
    );
  }

  closeErrorReport = () => {
    this.setState(
      {
        showErrorPopup: false,
        showErrorReportFor: null
      }
    );
  }


    showErrorSummary = (metricURI) => {
      this.setState(
        {
          showSummaryPopup: true,
          showSummaryReportFor: metricURI
        }
      );
    }

    closeErrorSummary = () => {
      this.setState(
        {
          showSummaryPopup: false,
          showSummaryReportFor: null
        }
      );
    }

  render() {
    console.log(this.props)
    if ((this.props.location.hash !== "" & this.state.showErrorPopup === false) | (this.props.location.hash !== "" & this.state.showSummaryPopup === false))
    {
      return <Redirect push to={this.props.location.pathname} />;
    }
    let canPublish = "Not Ready to Publish";
    let displayColour = noteReadyToPublish;
    let canPublishClass = "text-left font-weight-bold";
    let percentageValue = (this.state.dimensionActual * 100).toFixed(2);
    if (this.state.dimensionActual >= this.state.dimensionExpected)
    {
      canPublish = "Ready to Publish";
      displayColour = readyToPublish;
      canPublishClass = "text-left font-weight-bold";
    }

    let metrics = null;
    metrics = (
      this.state.metricValues.map((metricDetails, index) =>
      {
        let actual = 0;
        let expected = 0;
        let actualLabel = "";
        let expectedLabel = "";
        if (metricDetails.type === "boolean")
        {
          expected = 100;
          expectedLabel = "True";
          if (metricDetails.target === 0)
          {
            expectedLabel = "False";
          }
          actual = 0;
          actualLabel = "False";
          if (metricDetails.actual === 1)
          {
            actualLabel = "True";
          }
          if (metricDetails.actual === metricDetails.target)
          {
            actual = 100;
          }
        }
        else {
          actual = Number(parseFloat(metricDetails.actual).toFixed(4)) * 100;
          actualLabel = "Observed : " + actual.toString() + "%";
          expected = Number(parseFloat(metricDetails.target).toFixed(4)) * 100;
          expectedLabel = "Threshold : " + expected.toString() + "%"
        }

        let color = "danger";
        if (actual >= expected) {
          color = "success";
        }
        return <Card className="w-100" key={index}><CardHeader><b>{metricDetails.label}</b></CardHeader><CardBody><Col>
          <Container ><Card ><CardBody><div className="progress-group-bars"><Progress className="progress-xs" color="info" style={{ 'height': '20px' }} value={expected} ><small className="justify-content-center d-flex position-absolute w-100"><font size="2" color="black">{expectedLabel}</font></small></Progress><Progress className="progress-xs" style={{ 'height': '20px' }} color={color} value={actual}><small className="justify-content-center d-flex position-absolute w-100"><font size="2" color="black">{actualLabel}</font></small></Progress></div></CardBody></Card>
          </Container>
        </Col>
          {metricDetails.qualityProblem !== "" ? <Row className="justify-content-md-center"><Col className="col-md-auto"><b><Link to={{ pathname: this.props.location.pathname, hash: "#" + metricDetails.metric.split("#")[1] }} onClick={() => this.showErrorReport(metricDetails.metric)}>SHOW PROBLEM REPORT</Link></b></Col><Col className="col-md-auto"><b><Link to={{ pathname: this.props.location.pathname, hash: "#" + metricDetails.metric.split("#")[1] }} onClick={() => this.showErrorSummary(metricDetails.metric)}>SHOW PROBLEM SUMMARY</Link></b></Col></Row> : null}
        </CardBody></Card>
      })
    );

    let display = null;
    if (this.props.datasetDetailsCache.length > 0) {
      display = (<div className="animated fadeIn">
        <Row >
          <Card className="w-100" >
            <CardHeader>
              <span className="h3">{this.props.datasetDetailsCache[0].datasetName} / {this.state.dimension}</span>
            </CardHeader>
          </Card>
        </Row>
        <Row >
          <Col className="col-3" >
            <Card >
              <CardHeader>
                {this.state.dimension} Dimension
            </CardHeader>
              <CardBody className="pb-0">
                <CardSubtitle className={canPublishClass} style={{ color: displayColour }}>{canPublish}</CardSubtitle>
                <p />
                <div style={{ width: '150px', margin: 'auto' }}>
                  <CircularProgressbar percentage={percentageValue} text={`${percentageValue}%`} styles={{ text: { fill: displayColour }, path: { stroke: displayColour } }} />
                </div>
                <div style={{ height: '20px' }} />
              </CardBody>
            </Card>
          </Col>
          <Col className="col-9">
            <Card >
              <CardHeader>
                Metrics Measured
            </CardHeader>
              <CardBody>
                <Row>
                  {metrics}
                  {this.state.showErrorPopup ? <ErrorReport datasetID={this.props.match.params.datasetID} metricURI={this.state.showErrorReportFor} onCloseButtonClick={this.closeErrorReport} /> : null}
                  {this.state.showSummaryPopup ? <SummaryReport datasetID={this.props.match.params.datasetID} metricURI={this.state.showSummaryReportFor} onCloseButtonClick={this.closeErrorSummary} /> : null}
                </Row>
              </CardBody>
            </Card>
          </Col>
        </Row>
      </div>
      );
    }
    else {
      display = (<Redirect to="/dashboard" />);
    }

    return (
      <React.Fragment>
        {display}
      </React.Fragment>
    );
  }
}

const mapStateToProps = (state, ownProps) => {
  let datasetID = Number(ownProps.match.params.datasetID);
  let datasetDetails = state.datasetDetailsCache.filter((dataset) => { if (dataset.datasetID === datasetID) { return dataset; } return null; });
  return (
    {
      datasetDetailsCache: datasetDetails
    }
  );
}

export default connect(mapStateToProps)(DimensionDetails);
