import React, { Component } from 'react';
import {
  TabPane,
  CardBody, Card, ListGroup,ListGroupItem,
  UncontrolledTooltip
} from 'reactstrap';
import { connect } from 'react-redux';
import { read } from '../../../services/tripleStoreAPIs/readFromTripleStore';
import { listOfAllMetricsFailedForResource } from '../../../services/tripleStoreAPIs/sparql/problemReport/listOfAllMetricsFailedForResource';
import LoadingSpinner from '../loading';
import LoadingFailed from '../loadingFailed';
import metricExceptionMapping from '../../../config/metricKnowledgeBaseMapping';
import exceptionInfoIconDetails from '../../../config/exceptionInfoIconDetails';

class ResourcesFailedForProblem extends Component {
  constructor(props) {
    super(props);
    this.state = {
      isloading: false,
      activeTab: 0
    }
    //console.log(this.props);
  }

  toggle = (tab,resource) => {
    let failedMetricsToReturn=[];
    let indexOfMetric = this.props.datasetDetailsCache[0].lastAssessmentMetrics.findIndex((assessment) => { return assessment.Metric.value === this.props.metricURI });
    let graph=this.props.datasetDetailsCache[0].lastAssessmentMetrics[indexOfMetric].ProblemGraph.value;
    let modelQueryPart = "";
    for (const [key, value] of Object.entries(metricExceptionMapping.knowledgeBaseMapping)) {
        let metric = "dqm:"+key;
        let exceptions = Object.keys(value).map((exception)=> "dqm-prob:"+exception);
        let listOfExceptions = exceptions.join(", ") ;
        modelQueryPart += `
        UNION
          {{GRAPH ?QualityGraph {?QualityProblemRef qpro:problematicThing ?ProblematicThing;
                             qpro:problemStructure ?ProblemStructure;
                                                     qpro:isDescribedBy ?IsDescribedBy}}.
          {GRAPH ?QualityGraph {?ProblematicThing qpro:exceptionDescription ?Exception}}.
          FILTER (?IsDescribedBy = ${metric}).
          FILTER (?Exception IN (${listOfExceptions})).
         }
        `
      }
      //console.log(modelQueryPart);

      this.setState({ isloading: true }, ()=>{
        read(listOfAllMetricsFailedForResource(graph,resource, modelQueryPart)).then((response)=>
      {
        if(response)
        {
          this.setState({
            isloading: false
          },()=>{
            if(response.results.bindings.length>0)
            {
              failedMetricsToReturn=[...new Set(response.results.bindings.map((metric) => {
                //if(this.props.metricURI!==metric.IsDescribedBy.value)
                //{
                  let metricString = metric.IsDescribedBy.value.split("#")[1];

                  let exceptionString = "";
                  if(metric.Exception!==undefined)
                  {
                    exceptionString = metric.Exception.value.split("#")[1];
                    if(exceptionString===undefined)
                    {
                      //exceptionString=" - "+metric.Exception.value;
                      exceptionString=metric.Exception.value;
                    }
                    else {
                      //exceptionString = " - "+metric.Exception.value.split("#")[1];
                      exceptionString = metric.Exception.value.split("#")[1];
                    }

                  }

                  //return (metric.IsDescribedBy.value.split("#")[1]);
                  //return (metricString+exceptionString);
                  if(exceptionString!=="")
                  {
                    return (<React.Fragment>{metricString} - {exceptionString} <i className="fa fa-info-circle" id={exceptionString + "_toolTipIcon"}>  </i><UncontrolledTooltip placement="right" target={exceptionString + "_toolTipIcon"}>{exceptionInfoIconDetails.infoIcon[exceptionString]}</UncontrolledTooltip>&nbsp;</React.Fragment>)
                  }
                  else
                  {
                      return (<React.Fragment>{metricString}</React.Fragment>)
                  }

                //}
                //else
                //{
                //  return "";
                //}

              } ))];
              this.props.callBack(failedMetricsToReturn);
            }


          });
        }
        else {
          this.setState({
            isloading: false
          });
        }

      });
      })



    if (this.state.activeTab !== tab) {
      this.setState({
        activeTab: tab
      });
    }
  }

  render() {

    let indexOfMetric = this.props.datasetDetailsCache[0].lastAssessmentMetrics.findIndex((assessment) => { return assessment.Metric.value === this.props.metricURI });

    let returnList = null;
    returnList = this.props.listOfProblems.map((problem, index) => {

      let problemKey = Object.keys(problem);
      let resourcesEffectedAll=this.props.datasetDetailsCache[0].lastAssessmentMetrics[indexOfMetric].errorReport[problem[problemKey][1]].ProblematicThing[problemKey];
      let resourcesEffected=[...new Set(resourcesEffectedAll.map((resource) => (resource)))];

      if(resourcesEffected.length===1 & resourcesEffected[0]==="")
      {
        resourcesEffected=[];
      }

      let effectedCount = null;
      if (problem[problemKey][0]===0)
      {
        effectedCount="Unknown"
      }
      else {
        effectedCount=problem[problemKey][0];
      }
      let exception = problem[problemKey][1].split("#")[1];
      if(exception===undefined)
      {
        exception=problem[problemKey][1];
      }
      return  <TabPane tabId={index} key={index}>
        <p>Number of Resource or Triple effected by the selected Problematic Instance:  {effectedCount} </p>
        <p>Exception raised for selected Problematic Instance:  {exception} <i className="fa fa-info-circle" id={exception + "_toolTipIcon"}>  </i><UncontrolledTooltip placement="right" target={exception + "_toolTipIcon"}>{exceptionInfoIconDetails.infoIcon[exception]}</UncontrolledTooltip>&nbsp;</p>
        <Card style={{overflow: 'scroll', height:'20rem'}}>
        <p className="text-center font-weight-bold">List of distinct resource(s)</p>
            <CardBody>
              <ListGroup id="resourceList" role="tablist" className="flex-column align-items-start">

{resourcesEffected.length>0?
  resourcesEffected.map((resource,resourceIndex)=>{
  return (<ListGroupItem className="list-group-item list-group-item-action flex-column align-items-start" onClick={() => this.toggle(resourceIndex,resource)} action active={this.state.activeTab === resourceIndex} key={resourceIndex}> <p >{resource}</p></ListGroupItem>);
}) : null

}
                </ListGroup>
                </CardBody>
            </Card>
              </TabPane>
    });
    return (
      <React.Fragment>
      {this.state.isloading ? <LoadingSpinner /> : null}
      {returnList}
      </React.Fragment>
    );
  }
}

const mapStateToProps = (state, ownProps) => {
  let datasetID = Number(ownProps.datasetID);
  let datasetDetails = state.datasetDetailsCache.filter((dataset) => { if (dataset.datasetID === datasetID) { return dataset; } return null; });
  return (
    {
      datasetDetailsCache: datasetDetails
    }
  );
}


export default connect(mapStateToProps)(ResourcesFailedForProblem);
