//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component } from 'react';
import {
  Card,
  CardBody,
  CardHeader,
  Col,
  Progress,
  CardTitle,
  Container
} from 'reactstrap';
import { withRouter, Link } from 'react-router-dom';


class DimensionsQualityDetails extends Component {
  constructor(props) {
    super(props);
    //console.log("constructor");
    this.state = {
      assessmentCategory: [],
      assessmentDimensions : [],
      metricValues: []
    };
  }

  componentDidMount() {
    //console.log("componentDidMount");
    let categories = [...new Set(this.props.lastAssessmentMetrics.map((metric) => (metric.Category.value).split("#")[1]))];
    let dimensions = [...new Set(this.props.lastAssessmentMetrics.map((metric) => (metric.Dimension.value).split("#")[1]))];
    this.setState(
      {
        assessmentCategory: categories,
        assessmentDimensions : dimensions
      }, () => { this.prepareMetricDetails(); }
    );
  }

  prepareMetricDetails = () => {
    //console.log(this.state.assessmentCategory);
    //console.log(this.state.assessmentDimensions);
    let tempArray = [];
    tempArray = this.props.assessmentMetrics.map((metric) => {
      return (this.props.lastAssessmentMetrics.map((assessed) => {
        if (metric.metric === assessed.Metric.value & metric.assess) {
          return ({
            category: (assessed.Category.value).split("#")[1], dimension: (assessed.Dimension.value).split("#")[1], metric: assessed.Metric.value, value: assessed.Value.value, target: metric.target, type : (assessed.Value.datatype).split("#")[1]
          });
        }
      }));
    });
    this.setState(
      {
        metricValues: [...tempArray.flat().filter(item => item)]
      }
    );
  }

  render() {
    //console.log(this.state.metricValues);

    let metricDisplay = null;
    console.log(this.props)
    metricDisplay = (this.state.assessmentCategory.map((category) => {
      if(this.state.metricValues.findIndex((metrics)=>{return metrics.category===category})>=0)
      {
        return <Card key={category} className="w-50"><CardHeader><b>{category} Category</b></CardHeader><CardBody><Col>
          {
              this.state.assessmentDimensions.map((dimension, index)=>{
                let metricsInDimension=0;
                let overallExpected=0;
                let overallActual=0;
                  this.state.metricValues.forEach((metric)=>{
                    if(metric.category === category & metric.dimension === dimension)
                    {
                      metricsInDimension += 1;
                      if(metric.type==="boolean")
                      {
                        overallExpected += 1;
                        if(metric.value===metric.target)
                        {
                          overallActual += 1;
                        }
                        else {
                          overallActual += 0;
                        }
                      }
                      else {
                        overallActual += Number(parseFloat(metric.value).toFixed(4));
                        overallExpected += Number(parseFloat(metric.target).toFixed(4))
                      }
                    }
                  })
                  if(metricsInDimension>0)
                  {
                    let actual = Number(parseFloat(overallActual/metricsInDimension * 100).toFixed(2));
                    let expected = Number(parseFloat(overallExpected/metricsInDimension * 100).toFixed(2));
                    let color = "danger";
                    if (actual >= expected) {
                      color = "success";
                    }
                      //console.log(category + "-" + dimension + "-" + metricsInDimension + "-"+overallExpected+"-"+overallActual);
                      return <Link key={index} to={{ pathname: this.props.location.pathname + "/" + dimension, color:{color}, percentage:{actual} }}><Container ><Card key={index}><CardBody><CardTitle className="dimensionContainer"><b>{dimension}</b></CardTitle><div className="progress-group-bars"><Progress className="progress-xs" color="info" style={{ 'height': '20px' }} value={expected} ><small className="justify-content-center d-flex position-absolute w-100"><font size="2" color="black">Threshold : {expected}%</font></small></Progress><Progress className="progress-xs" style={{ 'height': '20px' }} color={color} value={actual}><small className="justify-content-center d-flex position-absolute w-100"><font size="2" color="black">Observed : {actual}%</font></small></Progress></div></CardBody></Card></Container></Link>
                  }
              })
          }
        </Col></CardBody></Card>
      }

    }));
    return (
      <React.Fragment>
        {metricDisplay}
      </React.Fragment>
    );
  }
}

export default withRouter(DimensionsQualityDetails);
