//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component,  } from 'react';
import {
  Card,
  CardBody,
  CardHeader,
  Row,
  CardSubtitle,
  Col,
  Container
} from 'reactstrap';
import { connect } from 'react-redux';
import { Redirect } from 'react-router-dom';
import DataQualityOverTime from '../DataQualityOverTime';
import LoadingSpinner from '../../Dashboard/loading';
import LoadingFailed from '../../Dashboard/loadingFailed';
import OneSpatialDimensionQualityDetails from './OneSpatialDimensionQualityDetails';
import PerformAnalysis from '../AnalyticsWidget/PerformAnalysis';
import CircularProgressbar from 'react-circular-progressbar';
import 'react-circular-progressbar/dist/styles.css';
const noteReadyToPublish = "rgba(255,0,0,0.6)";
const readyToPublish = "rgba(34,139,34,0.8)";


//Main page for 1Spatial data. Makes query requests and initializes data.
class OneSpatialDetails extends Component {
  state = {
  };
  constructor(props) {
    super(props);
    this.state = {
        analysisDropdownOpen: false,
        radioSelected: 2,
        datasetDetails: []
      }
  }


  loading = () => <div className="animated fadeIn pt-1 text-center">Loading...</div>

  render() {
    var sessionStore = JSON.parse(sessionStorage.getItem("PIPELINECACHE"));
    var pipelineNum = parseInt(this.props.match.params.datasetID)
    let percentageValue = 0;
    let display = null;
    var datasetDetails = sessionStore[pipelineNum]
    if(datasetDetails){
      percentageValue = Number.parseFloat(datasetDetails.lastAssessmentQuality * 100).toFixed(2);
      let canPublish = "Not Ready to Publish";
      let displayColour = noteReadyToPublish;
      let canPublishClass = "text-left font-weight-bold";

      if (datasetDetails.lastAssessmentQuality >= datasetDetails.expectedProgress) {
        canPublish = "Ready to Publish";
        displayColour = readyToPublish;
        canPublishClass = "text-left font-weight-bold";
      }

      display = (    <div className="animated fadeIn">
      <Row >
      <Card className="w-100" >
        <CardHeader>
        <span className="h3">{datasetDetails.datasetName}</span>

        </CardHeader>
        </Card>
      </Row>

            <Row >
              <Card className="w-25" >
                <CardHeader>
                  Current Data Quality
                </CardHeader>
                <CardBody className="pb-0">
                  <CardSubtitle className={canPublishClass} style={{ color: displayColour }}>{canPublish}</CardSubtitle>
                  <p/>
                  <div style={{ width: '150px', margin: 'auto' }}>
                    <CircularProgressbar percentage={percentageValue} text={`${percentageValue}%`} styles={{ text: { fill: displayColour }, path: { stroke: displayColour } }} />
                  </div>
                  <div style={{ height: '20px' }} />
                </CardBody>
              </Card>
             <DataQualityOverTime datasetID={datasetDetails.datasetID} is1Spatial={true} />
            </Row>

                    <OneSpatialDimensionQualityDetails lastAssessmentMetrics={datasetDetails.lastAssessmentMetrics} assessmentMetrics={datasetDetails.assessmentMetrics} dimensions={datasetDetails.dimensionData}/>
          </div>);
    }
    else {
    }


    return (

      <React.Fragment>
        {this.state.isloading ? <LoadingSpinner /> : null}
        {display}
      </React.Fragment>
    );
  }
}

//Adds the dataset to the cache
const mapStateToProps = async(state,ownProps) => {
  console.log(state)
  console.log(ownProps)
  if(state.pipelineCache){
    let datasetID = Number(ownProps.match.params.datasetID);
    let datasetDetails = await state.pipelineCache.filter((dataset)=>{if(dataset.datasetID===datasetID){return dataset;} return null; });
      return (
          {
              datasetDetailsCache: datasetDetails
          }
      );
  }
}

//Adds cache functions to props 
const mapDispatchToProps = (dispatch) => {
    return ({
        updateDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'UPDATE_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); },
        addDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'ADD_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); },
        removeDatasetDetailsCache: (datasetID) => { return dispatch({ type: 'REMOVE_DATASET_DETAILS_CACHE', payLoad: {datasetID:datasetID} }); },
        createDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'CREATE_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); }
    }
    );
}

export default connect(mapStateToProps, mapDispatchToProps) (OneSpatialDetails);