//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component } from 'react';
import {
  Card,
  CardBody,
  CardHeader,
  Row,
  CardSubtitle,
  Col,
  Progress,
  Container
} from 'reactstrap';
import { connect } from 'react-redux';
import { Redirect } from 'react-router-dom';
import CircularProgressbar from 'react-circular-progressbar';
import 'react-circular-progressbar/dist/styles.css';
import { Link } from 'react-router-dom';
const noteReadyToPublish = "rgba(255,0,0,0.6)";
const readyToPublish = "rgba(34,139,34,0.8)";


class OneSpatialDimensionDetails extends Component {

  constructor(props) {
    super(props);
    this.state = {
      dataset : JSON.parse(sessionStorage.getItem("PIPELINECACHE")),
      metrics: [],
      percentage: 0,
      dimension: this.props.match.params.dimension,
      categories: [],
      metricValues: [],
      categoryValues: []
    }
  }

  componentDidMount() {
    let metrics = [...new Set(this.state.dataset[Number(this.props.match.params.datasetID)].dimensionData.map((metric) => (metric.result.Metric.value)))];
    this.setState(
      {
        assessmentMetrics: metrics,
      }, () => { this.prepareMetricDetails(); }
    );
  }

  prepareMetricDetails = () => {
    //makes an object for each metric with their dimenions and value
    let tempArrayTwo = [];
    this.state.dataset[Number(this.props.match.params.datasetID)].assessmentMetrics.map((metric) => {
      (this.state.dataset[Number(this.props.match.params.datasetID)].lastAssessmentMetrics.map((assessed) => {
        if (((metric.metricURI).split("/")[4])+"-instance" === (assessed.result.MetricUri.value).split("/")[5]) {
            var item = {
             dimension: (metric.dimension).split("/")[4],
             metric: ((assessed.result.MetricUri.value).split("/")[5]), 
             value: assessed.result.Value.value, 
             target: 0.95,
             comment: metric.comment
            }
            tempArrayTwo.push(item)
        }
      }));
    });
    this.setState(
      {
        metricValues: tempArrayTwo,
      }
    );
  }

  render() {
    let actual = 0
    let canPublish = "Not Ready to Publish";
    let displayColour = noteReadyToPublish;
    let canPublishClass = "text-left font-weight-bold";
    let metricValues = this.state.metricValues
    let metricsInDimension=0;
    let overallExpected=0;
    let overallActual=0;
    let metrics=[];
    this.state.metricValues.forEach((metric)=>{
      if(metric.dimension+"-instance" === this.props.match.params.dimension)
        {
          metricsInDimension += 1;
          if(Number(parseFloat(metric.value).toFixed(4)) == -Infinity){
            overallActual += 0
          }else{
            overallActual += Number(parseFloat(metric.value).toFixed(4));
          }
          overallExpected += Number(parseFloat(metric.target).toFixed(4))
          metrics.push(metric)
        }
      })
      if(metricsInDimension>0)
      {
        actual = Number(parseFloat(overallActual/metricsInDimension * 100).toFixed(2));
        if(actual == -Infinity){
          actual = 0
        }
        let expected = Number(parseFloat(overallExpected/metricsInDimension * 100).toFixed(2));
        console.log(actual)
        console.log(expected)
        if (actual >= expected) {
          canPublish = "Ready to Publish";
          displayColour = readyToPublish;
          canPublishClass = "text-left font-weight-bold";
        }
          //console.log(category + "-" + dimension + "-" + metricsInDimension + "-"+overallExpected+"-"+overallActual);
      }

    //Creates a react component for each metric and their value
    metrics = (
      metrics.map((metricDetails, index) =>
      {
        let actual = 0;
        let expected = 0;
        let actualLabel = "";
        let expectedLabel = "";
        let color = "danger"

        actual = Number(parseFloat(metricDetails.value).toFixed(4)) * 100;
        if(actual == -Infinity){
          actual = 0
        }
        actualLabel = "Observed : " + actual.toString() + "%";
        expected = Number(parseFloat(metricDetails.target).toFixed(4)) * 100;
        expectedLabel = "Threshold : " + expected.toString() + "%"

        if (actual >= expected) {
          color = "success";;
        }
        var metricName = (metricDetails.metric).split("-")
        metricName = metricName[0] + "-" + metricName[1] + "-"  + metricName[2]
        return <Card className="w-100" key={index}><CardHeader><b>{metricName}</b></CardHeader><CardBody><Col>
          <b>{metricDetails.comment}</b><br></br>
          &nbsp;
          <Container ><Card ><CardBody><div className="progress-group-bars"><Progress className="progress-xs" color="info" style={{ 'height': '20px' }} value={expected} ><small className="justify-content-center d-flex position-absolute w-100"><font size="2" color="black">{expectedLabel}</font></small></Progress><Progress className="progress-xs" style={{ 'height': '20px' }} color={color} value={actual}><small className="justify-content-center d-flex position-absolute w-100"><font size="2" color="black">{actualLabel}</font></small></Progress></div></CardBody></Card>
          </Container>
        </Col>
        </CardBody></Card>
      })
    );

    let display = null;
      display = (<div className="animated fadeIn">
        <Row >
          <Card className="w-100" >
            <CardHeader>
              <span className="h3">1Spatial / {(this.state.dimension).split("-")[0]}</span>
            </CardHeader>
          </Card>
        </Row>
        <Row >
          <Col className="col-3" >
            <Card >
              <CardHeader>
                {(this.state.dimension).split("-")[0]} Dimension
            </CardHeader>
              <CardBody className="pb-0">
                <CardSubtitle className={canPublishClass} style={{ color: displayColour }}>{canPublish}</CardSubtitle>
                <p />
                <div style={{ width: '150px', margin: 'auto' }}>
                  <CircularProgressbar percentage={actual} text={`${actual}%`} styles={{ text: { fill: displayColour }, path: { stroke: displayColour } }} />
                </div>
                <div style={{ height: '20px' }} />
              </CardBody>
            </Card>
          </Col>
          <Col className="col-9">
            <Card >
              <CardHeader>
                Metrics Measured
            </CardHeader>
              <CardBody>
                <Row>
                  {metrics}
                </Row>
              </CardBody>
            </Card>
          </Col>
        </Row>
      </div>
      );
    return (
      <React.Fragment>
        {display}
      </React.Fragment>
    );
  }
}

export default OneSpatialDimensionDetails;