//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component } from 'react';
import {
  Card,
  CardBody,
  CardHeader,
  Col,
  Progress,
  CardTitle,
  Container
} from 'reactstrap';
import { withRouter, Link } from 'react-router-dom';


class OneSpatialDimensionQualityDetails extends Component {
  constructor(props) {
    super(props);
    //console.log("constructor");
    this.state = {
      dataset : JSON.parse(sessionStorage.getItem("PIPELINECACHE")),
      pipelineNum  : parseInt(this.props.match.params.datasetID),
      assessmentMetrics: [],
      assessmentDimensions : [],
      dimensionValues: [],
      metricValues: [],
      dimensionsNames: [],
      categories: [],
      categoryValues: []
    };
  }

  componentDidMount() {
    console.log(this.props)
    //console.log("componentDidMount");
    //makes arrays for each metric, dimension and dimension names
    var sessionStore = JSON.parse(sessionStorage.getItem("PIPELINECACHE"));
    var pipelineNum = parseInt(this.props.match.params.datasetID)
    let category = [...new Set(sessionStore[pipelineNum].dimensionData.map((metric) => (metric.result.cat.value).split("/")[5]))];
    let metrics = [...new Set(sessionStore[pipelineNum].dimensionData.map((metric) => (metric.result.Metric.value)))];
    let dimensions = [...new Set(sessionStore[pipelineNum].dimensionData.map((metric) => (metric.result.dimInstance.value)))];
    let dimensionsNames = [...new Set(sessionStore[pipelineNum].dimensionData.map((metric) => (metric.result.dimInstance.value).split("/")[5]))];
    this.setState(
      {
        categories: category,
        assessmentMetrics: metrics,
        assessmentDimensions : dimensions,
        dimensionsNames : dimensionsNames
      }, () => { this.prepareMetricDetails(); }
    );
  }

  prepareMetricDetails = () => {
    //makes a list of dimenions
    let tempArray = [];
    (this.state.assessmentDimensions).forEach((dimension) => {
      var metrics = [];
      (this.state.dataset[this.state.pipelineNum].dimensionData).forEach((metricDetails) => {
        if(dimension == metricDetails.result.dimInstance.value){
          metrics.push((metricDetails.result.Metric.value).split("/")[5])
        }
      });
      tempArray.push(metrics)
    });
    //makes an object for each metric with their dimenions and value
    let tempArrayTwo = [];
    this.state.dataset[this.state.pipelineNum].assessmentMetrics.map((metric) => {
      (this.state.dataset[this.state.pipelineNum].lastAssessmentMetrics.map((assessed) => {
        if (((metric.metricURI).split("/")[4])+"-instance" === (assessed.result.MetricUri.value).split("/")[5]) {
            var item = {
             dimension: (metric.dimension).split("/")[4],
             metric: ((assessed.result.MetricUri.value).split("/")[5]), 
             value: assessed.result.Value.value, 
             target: 0.95
            }
            tempArrayTwo.push(item)
        }
      }));
    });
    let tempArrayThree = [];
    this.state.categories.map((cat) => {
      let dims = [];
      (this.state.dataset[this.state.pipelineNum].dimensionData.map((assessed) => {
        if (cat === (assessed.result.cat.value).split("/")[5]) {
          var contains = false
          dims.forEach((metric)=>{
            if(metric == (assessed.result.dimInstance.value).split("/")[5]){
              contains = true
            }
          })
          if(!contains){
            dims.push((assessed.result.dimInstance.value).split("/")[5])
          }
        }
      }));
      var item = {
         category:  (cat).split("-")[0],
         dimensions: dims, 
        }
        tempArrayThree.push(item)
    });
    this.setState(
      {
        dimensionValues: tempArray,
        metricValues: tempArrayTwo,
        categoryValues: tempArrayThree
      }
    );
  }

  render() {
    let metricDisplay = null;
    var color = "danger";
    var dimensionValues = this.state.dimensionValues
    var metricValues = this.state.metricValues
    var categoryValues = this.state.categoryValues

    //Makes a react component for each dimenion with their value percent with threshold
    metricDisplay = (categoryValues.map((category) => {
        return <Card key={category.category} className="w-50"><CardHeader><b>{category.category}</b></CardHeader><CardBody><Col>
          { 
              category.dimensions.map((dimension, index)=>{
                var values = dimensionValues[index]
                let metricsInDimension=0;
                let overallExpected=0;
                let overallActual=0;
                let metrics=[];
                this.state.metricValues.forEach((metric)=>{
                  if(metric.dimension+"-instance" === dimension)
                    {
                      metricsInDimension += 1;
                      if(Number(parseFloat(metric.value).toFixed(4)) == -Infinity){
                        overallActual += 0
                      }else{
                        overallActual += Number(parseFloat(metric.value).toFixed(4));
                      }
                      overallExpected += Number(parseFloat(metric.target).toFixed(4))
                      metrics.push(metric)
                    }
                  })
                  if(metricsInDimension>0)
                  {
                    let actual = Number(parseFloat(overallActual/metricsInDimension * 100).toFixed(2));
                    if(actual == -Infinity){
                      actual = 0
                    }
                    let expected = Number(parseFloat(overallExpected/metricsInDimension * 100).toFixed(2));
                    let color = "danger";
                    if (actual >= expected) {
                      color = "success";
                    }
                      //console.log(category + "-" + dimension + "-" + metricsInDimension + "-"+overallExpected+"-"+overallActual);
                    return <Link key={index} to={{ pathname: this.state.pipelineNum + "/" + dimension, color:{color}, state: {values}, percentage:{actual}, metrics:{metrics} }}><Container ><Card key={index}><CardBody><CardTitle className="dimensionContainer"><b>{(dimension).split("-")[0]}</b></CardTitle><div className="progress-group-bars"><Progress className="progress-xs" color="info" style={{ 'height': '20px' }} value={expected} ><small className="justify-content-center d-flex position-absolute w-100"><font size="2" color="black">Threshold : {expected}%</font></small></Progress><Progress className="progress-xs" style={{ 'height': '20px' }} color={color} value={actual}><small className="justify-content-center d-flex position-absolute w-100"><font size="2" color="black">Observed : {actual}%</font></small></Progress></div></CardBody></Card></Container></Link>                  
                  }
              })
          }
        </Col></CardBody></Card>

    }));
    return (
      <React.Fragment>
        {metricDisplay}
      </React.Fragment>
    );
  }
}

export default withRouter(OneSpatialDimensionQualityDetails);
