//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component } from 'react';
import {
  Col,
  Container,
  Button,
  ButtonDropdown,
  ButtonGroup,
  Card,
  CardBody,
  CardText,
  CardTitle,
  CardSubtitle,
  DropdownItem,
  DropdownMenu,
  DropdownToggle,
  Media
} from 'reactstrap';
import { connect } from 'react-redux';
import { withRouter, Link } from 'react-router-dom';
import CircularProgressbar from 'react-circular-progressbar';
import 'react-circular-progressbar/dist/styles.css';
import { getRecentAssessmentQuality } from '../../../services/tripleStoreAPIs/sparql/getRecentAssessmentQuality';
import { read } from '../../../services/tripleStoreAPIs/readFromTripleStore';
import { triggerAssessment } from '../../../services/luzzuFrameWorkAPIs/triggerAssessment';
import { cancelAssessment } from '../../../services/luzzuFrameWorkAPIs/cancelAssessment';
import LoadingSpinner from '../../Dashboard/loading';
import LoadingFailed from '../../Dashboard/loadingFailed';
import { updateDataset, getStatus } from '../../../services/datasetConfigDetails/datasetConfigDetails';
import { getStatusUpdate } from '../../../services/luzzuFrameWorkAPIs/getStatusUpdate';
import PipelineConnectDataset from '../PipelineConnectDataset';
import { get1SpatialAssessmentMetrics } from '../../../services/tripleStoreAPIs/sparql/get1SpatialAssessmentMetrics';
import { get1SpatialAssessmentData } from '../../../services/tripleStoreAPIs/sparql/get1SpatialAssessmentData';
import { get1SpatialDimensions } from '../../../services/tripleStoreAPIs/sparql/get1SpatialDimensions';
import { get1SpatialMappings } from '../../../services/tripleStoreAPIs/sparql/get1SpatialMappings';
import metricExceptionMapping from '../../../config/metricKnowledgeBaseMapping';
import uploadImage from '../../../assets/cloud-upload.png'

const noteReadyToPublish = "rgba(255,0,0,0.6)";
const readyToPublish = "rgba(34,139,34)";

class DataSetLarge extends Component {

  constructor(props) {
    super(props);
  }
  state = {
    dropdownOpen: false,
    dataSetLastAssessed: "Not Assessed Yet",
    canAssess: true,
    lastAssessmentQuality: 0.0,
    isloading: false,
    loadFailed: false,
    lastAssessmentMetrics: [],
    datasetID:this.props.datasetID,
    dataset: this.props.dataSetDetails,
    hasDataset: false,
    showGraphSelect: false
  };

  recall = 0;

  toggle = () => {
    this.setState({
      dropdownOpen: !this.state.dropdownOpen,
    });
  }


  componentDidMount() {
  }

  componentWillUnmount() {
    //console.log("Component Will  Unmount");
    //console.log(this.recall);
    clearInterval(this.recall);
    //console.log(this.recall);
  }


  checkWrapperServiceIsReachable =()=>
  {
    let isReachable = false;

  return new Promise((resolve)=>
    {
      this.setState({isloading:true},()=>{
        getStatus().then((response)=>{
          this.setState({isloading:false},()=>{
            if(response.status===200)
            {
              if(response.data.status==="OK")
              {
                isReachable=true;
                resolve(isReachable);
              }
              else
              {
                isReachable=false;
                resolve(isReachable);
              }
            }
            else
            {
              isReachable=false;
              resolve(isReachable);
            }
          });
        });
      });
    }
  );
  }

  //Makes SPARQL queries and adds data to dataset
  setup = async(graphName) =>
  {
    let datasetDetails = await this.initialize1Spatial();
    let metrics = await this.fetch1SpatialMetrics()
    let assessmentData = await this.fetch1SpatialAssessmentData(graphName)
    let dimensionData = await this.fetch1SpatialDimensions(graphName)
    let mappingData = await this.fetch1SpatialMappings()
    console.log(assessmentData)
    datasetDetails.assessmentMetrics = metrics
    datasetDetails.historicAssessmentData = assessmentData.historicAssessmentData
    datasetDetails.dataSetLastAssessed = assessmentData.dataSetLastAssessed
    datasetDetails.lastAssessmentQuality = assessmentData.lastAssessmentQuality
    datasetDetails.lastAssessmentMetrics = assessmentData.lastAssessmentMetrics
    datasetDetails.onBuildingsDimensions = assessmentData.onBuildings
    datasetDetails.dimensionData = dimensionData
    datasetDetails.mappingData = mappingData
    this.setState({dataset: datasetDetails, dimensionData: dimensionData})
    var hasCache = false;
    if(this.props.datasetDetailsCache){
      (this.props.datasetDetailsCache).forEach(function(dataset) {
        if(dataset.datasetID == 1){
          hasCache = true
        }
      }); 
    }
    if(!hasCache){
      await this.props.addPipelineCache(datasetDetails);
    }
    if(dimensionData){
      this.setState({ isloading: false })
    }
    return;
  }

  //Metric SPAQRL query
  fetch1SpatialMetricsFromTripleStore = () =>
  {
      return new Promise((resolve)=>{
        let spatialmetrics = [];
        read(get1SpatialAssessmentMetrics())
          .then((response) => {
            //console.log(response);
            if (response) {
              if (response.results.bindings.length > 0) {
                spatialmetrics = response.results.bindings.map((result,index) => {
                  return ({result:result});
                })
              }
            }
            resolve(spatialmetrics);
        });

      });
  }

  //Makes object array for metric data
  fetch1SpatialMetrics = async(graphName) =>
  {
    let definedMetrics = Object.keys(metricExceptionMapping.knowledgeBaseMapping);
    let qualityMetrics = [];
    let response = await this.fetch1SpatialMetricsFromTripleStore(graphName)
    let graphs = response;
    graphs.forEach((graph) => {
      let label = graph.result.label.value;
      let javaPackageName = "";
      let comment = graph.result.comment.value;
      let metricURI = graph.result.metric.value;
      let dataType = (graph.result.datatype.value).split("#")[1];
      let category = "";
      let dimension = graph.result.dimension.value;
      qualityMetrics.push(
        {
          label: label,
          comment: comment,
          javaPackageName: javaPackageName,
          category: category,
          dimension: dimension,
          metricURI: metricURI,
          dataType: dataType
        }
      );
    });
    return qualityMetrics
  }

  //Assessment SPARQL query
  fetch1SpatialAssessmentDataFromTripleStore = (graphName) =>
  {
      return new Promise((resolve)=>{
        let spatialAssessmentData = [];
        read(get1SpatialAssessmentData(graphName))
          .then((response) => {
            if (response) {
              if (response.results.bindings.length > 0) {
                spatialAssessmentData = response.results.bindings.map((result,index) => {
                  return ({result:result});
                })
              }
            }
            resolve(spatialAssessmentData);
        });

      });
  }

  //Makes object array for assessment data
  fetch1SpatialAssessmentData = async(graphName) =>
  {
    let definedMetrics = Object.keys(metricExceptionMapping.knowledgeBaseMapping);
    var assessmentData = [];
    var historics = [];
    var assessmentDate = "Not Assessed Yet";
    var assessmentQuality = 0.00;
    var lastAssessmentDate = ''
    var data = {
      dataSetLastAssessed: '',
      lastAssessmentQuality: '',
      lastAssessmentMetrics: '',
      historicAssessmentData: ''
    }
    var historicAssessments = {}
    var assessments = []
    let response = await this.fetch1SpatialAssessmentDataFromTripleStore(graphName)
    let graphs = response;
    //assessmentDate = (graphs[0].result.TimePeriod.value).split(".")[0]
    assessmentQuality = 0.00
    graphs.forEach((metricDetails) => {

      lastAssessmentDate = (metricDetails.result.maxTimePeriod.value).split(".")[0]

      assessmentDate = (metricDetails.result.TimePeriod.value).split(".")[0]
      if(!historicAssessments[assessmentDate]){
        historicAssessments[assessmentDate] = {
          dateAssessed: assessmentDate,
          assessmentQuality: 0.00,
          metricCount: 0
        }
      }
      if(metricDetails.result.Value.value != "-Infinity"){
        historicAssessments[assessmentDate].assessmentQuality += parseFloat(metricDetails.result.Value.value)
        historicAssessments[assessmentDate].metricCount++
      }
      else{
        historicAssessments[assessmentDate].assessmentQuality += 0
        historicAssessments[assessmentDate].metricCount++
      }
    });

    for (var property in historicAssessments) {
      historicAssessments[property].assessmentQuality = (historicAssessments[property].assessmentQuality/historicAssessments[property].metricCount).toFixed(4);
      historics.push(historicAssessments[property])
      data.dataSetLastAssessed = historicAssessments[property].dateAssessed
    };

    graphs.forEach((metricDetails) => {

      if((metricDetails.result.TimePeriod.value) === metricDetails.result.maxTimePeriod.value){
        assessmentData.push(
          metricDetails
        );
      }
    });

    historics.sort((a, b) => (a.dateAssessed > b.dateAssessed) ? 1 : -1)

    data.dataSetLastAssessed = lastAssessmentDate
    data.lastAssessmentQuality = historicAssessments[lastAssessmentDate].assessmentQuality
    data.historicAssessmentData = historics
    data.lastAssessmentMetrics = assessmentData
    return data
  }

  //Dimension SPARQL query
  fetch1SpatialDimensionsFromTripleStore = (graphName) =>
  {
      return new Promise((resolve)=>{
        let spatialDimensions = [];
        read(get1SpatialDimensions(graphName))
          .then((response) => {
            //console.log(response);
            if (response) {
              if (response.results.bindings.length > 0) {
                spatialDimensions = response.results.bindings.map((result,index) => {
                  return ({result:result});
                })
              }
            }
            resolve(spatialDimensions);
        });

      });
  }

  fetch1SpatialDimensions = async(graphName) =>
  {
    let definedMetrics = Object.keys(metricExceptionMapping.knowledgeBaseMapping);
    let qualityMetrics = [];
    let response = await this.fetch1SpatialDimensionsFromTripleStore(graphName)
    return response
  }

  fetch1SpatialMappingsFromTripleStore = () =>
  {
      return new Promise((resolve)=>{
        let spatialmappings = [];
        read(get1SpatialMappings())
          .then((response) => {
            if (response) {
              if (response.results.bindings.length > 0) {
                spatialmappings = response.results.bindings.map((result,index) => {
                  return ({result:result});
                })
              }
            }
            resolve(spatialmappings);
        });

      });
  }

  //Makes object array for metric data
  fetch1SpatialMappings = async() =>
  {
    let definedMetrics = Object.keys(metricExceptionMapping.knowledgeBaseMapping);
    let response = await this.fetch1SpatialMappingsFromTripleStore()
    let graphs = response;
    return graphs
  }

  //Creates dataset object
  initialize1Spatial = () => {
    let requestBody = {
      datasetID: this.props.pipelineNum,
      datasetName: "1Spatial",
      fileName: "",
      datasetPLD: "uri:1Spatial",
      isSparqlEndPoint: false,
      sparqlEndPoint: "",
      lastAssessmentRequestID: "",
      knowledgeBaseID: "",
      expectedProgress: 0.95,
      assessmentMetrics: '',
      dataSetLastAssessed: '',
      historicAssessmentData: '',
      lastAssessmentMetrics: '',
      lastAssessmentQuality: '',
      dimensionData: '',
      mappingData: '',
      onBuildingsDimensions: ''
    }
    return requestBody
  }

  toggleAlertModel = () => {
    this.setState({
      loadFailed: false
    });
  }

  clicked = () => {
    this.setState({
      showGraphSelect: true
    });
  }

  closed = () => {
      if(this.state.showGraphSelect){
        //If it is already open, then close it and set connectDataSet as null
        this.setState(prevState => ({
            showGraphSelect: !prevState.showGraphSelect
        }));
      }
      else {
        this.setState(prevState => ({
            showGraphSelect: !prevState.showGraphSelect
        }));
      }
  }

  connectGraph = (graph) => {
      if(this.state.showGraphSelect){
        //If it is already open, then close it and set connectDataSet as null
        this.setup(graph)
        this.setState(prevState => ({
            showGraphSelect: !prevState.showGraphSelect
        }));
      }
      else {
        this.setState(prevState => ({
            showGraphSelect: !prevState.showGraphSelect
        }));
      }
  }

  openConfig = () => {
    this.props.onConfigClick(this.props.dataSetDetails);
  }

  render() {
    var sessionStore = JSON.parse(sessionStorage.getItem("PIPELINECACHE"));
    let camelize = (str) => {
      return str.replace(/(?:^\w|[A-Z]|\b\w)/g, (word, index) => {
        return index === 0 ? word.toLowerCase() : word.toUpperCase();
      }).replace(/\s+/g, '');
    }

    let canPublish = "Not Ready to Publish";
    let displayColour = noteReadyToPublish;
    let canPublishClass = "text-left font-weight-bold";
    let percentageValue = 0;
    let progressBar = null;
    let link = null

    if(sessionStore){
      if(sessionStore[this.props.pipelineNum]){
        percentageValue = Number.parseFloat(sessionStore[this.props.pipelineNum].lastAssessmentQuality * 100).toFixed(2);
      //console.log(percentageValue);

        if ((sessionStore[this.props.pipelineNum]).lastAssessmentQuality >= (sessionStore[this.props.pipelineNum]).expectedProgress) {
          canPublish = "Ready to Publish";
          displayColour = readyToPublish;
          canPublishClass = "text-left font-weight-bold";
        }
      }

      if(this.props.link == 'onespatial'){
        progressBar =  (<CircularProgressbar percentage={percentageValue} text={`${percentageValue}%`} background backgroundPadding={6} styles={{ background: { fill: '#3e98c7' }, text: { fill: displayColour }, path: { stroke: displayColour }, trail: { stroke: 'transparent' } }} />);
      }

    var styleCard = "text-white bg-info text-center w-100 h-90 pipelineTiles"
    var styleText = "text-value font-weight-bold text-dark text-left text-truncate"
    if(this.props.link !== 'onespatial'){
      styleCard = "text-white bg-info text-center w-100 h-90 pipelineTiles "
      styleText = "text-value font-weight-bold text-dark text-center text-truncate cardTextDown"
    }
      if(sessionStore[this.props.pipelineNum])
      {
         link =  (<Link to={{ pathname: this.props.link+ "/" +this.props.pipelineNum }}>
            <CardTitle className={styleText}>{this.state.datasetID}</CardTitle>
            <div style={{ width: '150px', margin: 'auto' }}>
              {progressBar}
            </div>
            <br />
          </Link>);
      }
      else if(this.props.link !== 'onespatial')
      {
         link =  (<Link to={{ pathname: this.props.link }}>
            <CardTitle className={styleText}>{this.state.datasetID}</CardTitle>
            <div style={{ width: '150px', margin: 'auto' }}>
              {progressBar}
            </div>
            <br />
          </Link>);
      }
      else{
        if(this.state.showGraphSelect){
          link = (<PipelineConnectDataset onClose={this.closed} onNext={this.props.onNext} parentCallback= {this.connectGraph}/>)
        }
        else{
          link = (      <div>               
                          <CardTitle></CardTitle>
                            <CardSubtitle></CardSubtitle>
                            <Media style={{ height: '150px', width: '150px', margin: 'auto' }} src={uploadImage} alt="Upload Image" />
                            <CardText className="text-value text-dark">CONNECT</CardText>
                            <CardText className="text-dark">a dataset</CardText>
                        </div>)
        }
      }
    }
    return (
      <Col xs="12" sm="6" lg="4">
        <Container style={{ margin: 'auto' }} className=" w-100 h-90 pipelineTiles" onClick= {this.clicked}>
          <Card className={styleCard} >
            <CardBody className="pb-0">
              {link}
            </CardBody>
          </Card>
        </Container>
        {this.state.isloading ? <LoadingSpinner /> : null}
        {this.state.loadFailed ? <LoadingFailed clickHandler={this.toggleAlertModel} /> : null}
      </Col>
    );
  }
}

const mapStateToProps = (state) => {
    return (
        {
            datasetDetailsCache: state.datasetDetailsCache
        }
    );
}


const mapDispatchToProps = (dispatch) => {
    return ({
        updateDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'UPDATE_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); },
        addDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'ADD_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); },
        removeDatasetDetailsCache: (datasetID) => { return dispatch({ type: 'REMOVE_DATASET_DETAILS_CACHE', payLoad: {datasetID:datasetID} }); },
        createDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'CREATE_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); },
        createPipelineCache: (pipelineDetails) => { return dispatch({ type: 'CREATE_PIPELINE_CACHE', payLoad: pipelineDetails }); },
        addPipelineCache: (pipelineDetails) => { return dispatch({ type: 'ADD_PIPELINE_CACHE', payLoad: pipelineDetails }); }
    }
    );
}
export default connect(mapStateToProps, mapDispatchToProps)(DataSetLarge);
