//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component } from 'react';
import { connect } from 'react-redux';
import { Row, Col, Dropdown, DropdownToggle, DropdownMenu, DropdownItem, InputGroup, Input, Card, Container, CardBody, CardText, CardTitle,} from 'reactstrap';
import ConnectDataSet from '../Dashboard/ConnectDataSet/ConnectDataSet';
import DataSetLarge from './DataSetLarge/DataSetLarge'
import { read } from '../../services/tripleStoreAPIs/readFromTripleStore';
import { getAllDataset, removeDataset,getDataset } from '../../services/datasetConfigDetails/datasetConfigDetails';
import { deleteGraph } from '../../services/tripleStoreAPIs/deleteGraphFromTripleStore';
import DatasetConnectConfigModelWindow from '../Dashboard/ConnectDataSet/DatasetConnectConfigModelWindow';
import PipelineConnectDatasetOptions from './PipelineConnectDatasetOptions';
import LoadingSpinner from '../Dashboard/loading';
import LoadingFailed from '../Dashboard/loadingFailed';
import properties from '../../config/dashboardProperties';
import TripleStoreUploadError from '../Dashboard/tripleStoreUploadError';

const limit = properties.dataQualityOverTimeBarCount;

class Pipeline extends Component {

  constructor(props) {
    super(props);
    if (sessionStorage.getItem("LIMIT")===null) {
      sessionStorage.setItem("LIMIT",limit );
    }

  }

    state = {
        inCache:false,
        datasetDetails: [],
        dropdownOpen: false,
        selectedView: "Large Title View",
        searchString: "",
        showConfigModal: false,
        showOptionModel: false,
        connectDataSet: null,
        graph: 'http://data.example.com/1spatialassessment/quality-graph',
        a: false
    };


    toggleDropDown = () => {
        this.setState(prevState => ({
            dropdownOpen: !prevState.dropdownOpen
        }));
    }

    toggleAlertModel = () => {
        this.setState({
            loadFailed: false
        });
    }

    toggleBetweenConfigAndOptionModel = () =>{
      this.setState(prevState => ({
          showOptionModel: !prevState.showOptionModel,
      }));
    }

    toggleConfigModel = () => {
      if(this.state.showConfigModal){
        //If it is already open, then close it and set connectDataSet as null
        this.setState(prevState => ({
            showConfigModal: !prevState.showConfigModal,
            connectDataSet : null
        }));
      }
      else {
        this.setState(prevState => ({
            showConfigModal: !prevState.showConfigModal
        }));
        this.toggleOptionModel()
      }
    }

    toggleOptionModel = () => {
      //console.log(uploadError);
      if(this.state.showOptionModel){
        //If it is already open, then close it and set connectDataSet as null
        this.setState(prevState => ({
            showOptionModel: !prevState.showOptionModel,
            connectDataSet : null
        }));
      }
      else {
        this.setState(prevState => ({
            showOptionModel: !prevState.showOptionModel
        }));
      }

    }

    toggleTripleStoreErrorModel = () => {
        this.setState({
            showTripleStoreError: false,
            graphName:""
        });
    }

    componentWillUnmount(){
    }

    componentDidMount() {
      //If there are no datasetDetails in Cache, then Call API to fetch the data
      //and the update the sessionStorage Cache.
      this.setup()
    }

  setup = async() =>
  {
    if(sessionStorage.getItem("DATASETCACHE")===null){
      await this.props.createDatasetDetailsCache([]);
      if(this.props.datasetDetailsCache && this.props.datasetDetailsCache.length <= 1)
        {
          await this.setState({ isloading: true }, () => {
              getAllDataset().then((response) => {
                  if (response.status === 200) {
                      this.setState({
                          isloading: false,
                          datasetDetails: response.data.datasetDetails,
                          loadFailed: false
                      }, ()=>{
                          this.state.datasetDetails.map((dataset) => {
                            this.props.addDatasetDetailsCache(dataset)
                          })
                      });
                  }
                  else {
                      this.setState({
                          isloading: false,
                          datasetDetails: [],
                          loadFailed: true
                      });
                  }
              });
          });
        }
    }
    if(sessionStorage.getItem("PIPELINECACHE")===null){
      await this.props.createPipelineCache([]);
    }
    return;
  }
    componentDidUpdate() {

    }

    toggleConfigModel = () => {
      if(this.state.showConfigModal){
        //If it is already open, then close it and set connectDataSet as null
        this.setState(prevState => ({
            showConfigModal: !prevState.showConfigModal,
            connectDataSet : null
        }));
      }
      else {
        this.setState(prevState => ({
            showConfigModal: !prevState.showConfigModal
        }));
      }
    }

    addDataSetComponent = async (dataSetDetails) => {

      await this.props.addDatasetDetailsCache(dataSetDetails);
      //console.log(this.props.datasetDetailsCache);
      this.setState(
        {
          datasetDetails: [...this.props.datasetDetailsCache]
        }
      );
      if(this.state.searchString!=="")
      {
        this.searchHandler();
      }
    }

    loading = () => <div className="animated fadeIn pt-1 text-center">Loading...</div>

    render() {
        let sessionStore = JSON.parse(sessionStorage.getItem("PIPELINECACHE"))
        let display = (            
            <div className="animated fadeIn">
                {this.state.showTripleStoreError?<TripleStoreUploadError clickHandler={this.toggleTripleStoreErrorModel} graphName={this.state.graphName} />:null}
                <br />

                <br />
                {this.state.showConfigModal ? <DatasetConnectConfigModelWindow onClose={this.toggleConfigModel} onNext={this.toggleBetweenConfigAndOptionModel} currentState={this.state.connectDataSet}/> : null}
                {this.state.showOptionModel && sessionStore[0] ? <PipelineConnectDatasetOptions onBack={this.toggleBetweenConfigAndOptionModel} onSubmit={this.toggleOptionModel} currentState={this.state.connectDataSet} whenUpdateExisting={this.updateDataSetComponent} whenAddNew={this.addDataSetComponent}/>: null}
                <Row>
                    <DataSetLarge className="prime2" key={0} pipelineNum={0} dataSetDetails={''} datasetID={'Prime2'} link={'onespatial'} onNext={this.toggleBetweenConfigAndOptionModel} />
                    <DataSetLarge className="mrds" key={1} pipelineNum={0} dataSetDetails={''} datasetID={'MRDS'} link={'onespatial'} onNext={this.toggleBetweenConfigAndOptionModel}/>
                    <DataSetLarge className="linked" key={2} pipelineNum={2} dataSetDetails={''} datasetID={'Linked Data'} link={'linkeddata'} onNext={this.toggleBetweenConfigAndOptionModel}/>
                    <Col>
                      <Container style={{ margin: 'auto' }} className="front" >
                        <Card className="text-white bg-info text-center front" >
                          <CardBody className="pb-0">
                          </CardBody>
                        </Card>
                      </Container>
                    </Col>
                </Row>
            </div>)
        return (
          <React.Fragment>
            {this.state.isloading ? <LoadingSpinner /> : null}
            {this.state.loadFailed ? <LoadingFailed clickHandler={this.toggleAlertModel} /> : null}
            {display}
          </React.Fragment>
        );
    }
}

const mapStateToProps = (state) => {
    return (
        {
            datasetDetailsCache: state.datasetDetailsCache
        }
    );
}

const mapDispatchToProps = (dispatch) => {
    return ({
        updateDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'UPDATE_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); },
        addDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'ADD_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); },
        removeDatasetDetailsCache: (datasetID) => { return dispatch({ type: 'REMOVE_DATASET_DETAILS_CACHE', payLoad: {datasetID:datasetID} }); },
        createDatasetDetailsCache: (datasetDetails) => { return dispatch({ type: 'CREATE_DATASET_DETAILS_CACHE', payLoad: datasetDetails }); },
        createPipelineCache: (pipelineDetails) => { return dispatch({ type: 'CREATE_PIPELINE_CACHE', payLoad: pipelineDetails }); },
        addPipelineCache: (pipelineDetails) => { return dispatch({ type: 'ADD_PIPELINE_CACHE', payLoad: pipelineDetails }); }
    }
    );
}
export default connect(mapStateToProps, mapDispatchToProps)(Pipeline);
