//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component} from 'react';
import { Bar, Line } from 'react-chartjs-2';
import {
  Card,
  CardBody,
  CardHeader,
  ButtonDropdown,
  ButtonGroup,
  DropdownItem,
  DropdownMenu,
  DropdownToggle,
  Media,
  Modal,
  ModalHeader,
  ModalBody,
  ModalFooter,
  Button,
  FormGroup,
  Form,
  Label,
  Input,
} from 'reactstrap';
import { connect } from 'react-redux';
import { CustomTooltips } from '@coreui/coreui-plugin-chartjs-custom-tooltips';
import { getAllAssessmentDates } from '../../services/tripleStoreAPIs/sparql/getAllAssessmentDates';
import { getAssessmentQuality } from '../../services/tripleStoreAPIs/sparql/getAssessmentQuality';
import { read } from '../../services/tripleStoreAPIs/readFromTripleStore';
import LoadingSpinner from '../DataSetDetails/loading';
import LoadingFailed from '../DataSetDetails/loadingFailed';


const options = {
  tooltips: {
    enabled: false,
    custom: CustomTooltips
  },
  scales: {
    xAxes: [{
        barThickness: 40,
        scaleLabel: {
        display: true,
        labelString: "Dimensions",

      }
    }],
    yAxes:[
      {
        ticks :{
          max: 100,
          min: 0,
          stepSize: 20
        },
        scaleLabel: {
        display: true,
        labelString: "Data Quality"

      }
      }
    ]
},
legend: {
    display: false
},

  maintainAspectRatio: false
}

class ReportDimensionBar extends Component{

  constructor(props) {
    super(props);
    this.state = {
      assessmentDates:[],
      assessmentQualities:[],
      qualityStatus :[],
      isloading : false,
      loadFailed :  false,
      dropdownOpen: false,
      popupOpen: false,
      tempLimit: sessionStorage.getItem("LIMIT"),
      dimensions: ''
    };


  }

  togglePopup = () => {
      this.setState(prevState => ({
          popupOpen: !prevState.popupOpen
      }));
  }

  onChangeLimit = (event) =>
  {
    this.setState(
        { tempLimit: event.target.value }
    );
  }

  saveLimit = () =>
  {
    //console.log(this.state.tempLimit);
    sessionStorage.setItem("LIMIT",this.state.tempLimit );
    this.togglePopup();
  }

componentDidMount(){
}


toggleAlertModel = () => {
    this.setState({
        loadFailed: false
    });
}

toggle = () => {
  this.setState({
    dropdownOpen: !this.state.dropdownOpen,
  });
}

dimensionColour = (value) => {
  let status="rgba(255,99,132,0.6)";
  if (value >= 95.00) {
    status="rgba(0,255,0,0.6)";
  }
  return status;
}

render ()
{
  let dimBar = null
  if(this.props.dimensions.dimensions){
    let bar = {
      labels: this.props.dimensions.dimensions.map((data)=>{ return(data.dim)}),
      datasets: [
        {
          label: '',
          backgroundColor: this.props.dimensions.dimensions.map((data)=>{return(this.dimensionColour(data.value))}),

          data:  this.props.dimensions.dimensions.map((data)=>{return(data.value)}) ,
        },
      ],
    };
    dimBar = (<Bar data={bar} options={options} />)
  }

  let model = (<Modal isOpen={true} className="modal-dialog-centered">
                  <ModalHeader style={{ margin: 'auto' }}>Configure Settings</ModalHeader>
                  <ModalBody>
                      <Form>
                          <FormGroup>
                              <Label>Dataset Name:</Label>
                               <Input type="text" name="limitInput" id="limitInput" ref="limitInput" value={this.state.tempLimit} onChange={this.onChangeLimit} />
                          </FormGroup>
                          </Form>
                      </ModalBody>
                      <ModalFooter>
                          <Button color="secondary" onClick={this.togglePopup}>Close</Button>
                          <Button color="primary" onClick={this.saveLimit}>Save</Button>{' '}
                      </ModalFooter>
                  </Modal>
                        );

return(
  <Card className="w-75 dim-bar-custom-height">
  {this.state.popupOpen?model:null}
    <CardHeader>
      Dimension Qualities ( {this.props.standard} )
      <ButtonGroup className="float-right">
        <ButtonDropdown id='card1' isOpen={this.state.dropdownOpen} toggle={this.toggle}>
          <DropdownToggle caret className="p-0" color="black">
            <Media className="icon-settings"></Media>
          </DropdownToggle>
          <DropdownMenu right>
            <DropdownItem onClick={this.togglePopup}>Config</DropdownItem>
          </DropdownMenu>
        </ButtonDropdown>
      </ButtonGroup>
    </CardHeader>
    <CardBody>

    {this.state.isloading ? <LoadingSpinner /> : null}
    {this.state.loadFailed ? <LoadingFailed clickHandler={this.toggleAlertModel} /> : null}
        {dimBar}
    </CardBody>
  </Card>
);

}

}

export default ReportDimensionBar;