//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component } from 'react';
import {  Col, Container, Row } from 'reactstrap';
import {Radar} from 'react-chartjs-2';
import {
  AppBreadcrumb,
  AppFooter,
  AppHeader,
  AppSidebar,
  AppSidebarFooter,
  AppSidebarForm,
  AppSidebarHeader,
  AppSidebarMinimizer,
  AppSidebarNav,
} from '@coreui/react';
import ReportOverTime from './ReportOverTime';
import ReportDimensionBar from './ReportDimensionBar';

class ReportList extends Component {
  constructor(props) {
    super(props);
    //console.log("constructor");
    this.state = {
      dimensions: [],
      linked: false,
      standard: "Linked Data",
      allDims: true,
      disabledDims: true,
      checkedDimList: [],
      quality: 0
    };
  }

  componentDidMount(){
    var sessionStore = JSON.parse(sessionStorage.getItem("PIPELINECACHE"));
    if(sessionStore && sessionStore.length > 0){
      this.prepareDimDetails();
    }
  }

  handleStandardChange = () => {
    if(this.state.linked){
      this.setState({ linked:false, standard: "Linked Data", allDims: true}, () => {
        this.prepareDimDetails();
        this.reportOverTime.setDatasetData();
      });
    }else{
      this.setState({ linked:true, standard: "ISO 19157", allDims: true}, () => {
        this.prepareDimDetails();
        this.reportOverTime.setDatasetData();
      });
    }
  }

  resetCheckboxes = () => {
    var array = this.state.dimensions
    array.dimensions.map((dim) =>
    {
      dim.checked = false
    })
    this.setState({dimensions: array})
  }

  handleDimChange = () => {
    if(this.state.allDims){
      this.resetCheckboxes()
      this.setState({ allDims:false, disabledDims: false, checkedDimList: []}, () => {
      });
    }else{
      this.setState({ allDims:true, disabledDims: true, quality: 0}, () => {
      });
    }
  }

  createCheckedDimList = (event) => {
    var arrayDimList = this.state.checkedDimList
    var contains = false
    var arrayDimensions = this.state.dimensions
    arrayDimensions.dimensions.map((dim) =>
    {
      if(event.target.name === dim.dim){
        if(dim.checked)
          dim.checked = false
        else
          dim.checked = true
      }
    })
    arrayDimList.forEach((dim, index)=>{
      if(dim.name == event.target.name){
        contains = true
        arrayDimList.splice(index, 1);
      }
    })
    if(!contains)
    {
      arrayDimList.push({name: event.target.name, value: event.target.value})
    }
    this.setState({checkedDimList: arrayDimList, dimensions: arrayDimensions})
  }

  prepareDimDetails = () => {
    var sessionStore = JSON.parse(sessionStorage.getItem("PIPELINECACHE"));
    let tempArray = [];
    sessionStore.map((pipe) => {
      let dims = [];
      (pipe.mappingData.map((dim) => {
        var contains = false
        dims.forEach((metric)=>{
          if(!this.state.linked){
            if(metric.dim == (dim.result.onespatial.value).split("#")[1]){
              contains = true
            }
          }else{
            if(metric.dim == (dim.result.linked.value).split("/")[4]){
              contains = true
            }
          }
        })
        var dimItem = {
          dim: '',
          value: '',
          checked: false
        }
        if(!contains){
          if(!this.state.linked){
            dimItem.dim = (dim.result.onespatial.value).split("#")[1]
            dims.push(dimItem)
          }else{
            dimItem.dim = (dim.result.linked.value).split("/")[4]
            dims.push(dimItem)
          }
        }
      }));
      let tempArrayTwo = [];
      sessionStore[0].assessmentMetrics.map((metric) => {
        (sessionStore[0].lastAssessmentMetrics.map((assessed) => {
          if (((metric.metricURI).split("/")[4])+"-instance" === (assessed.result.MetricUri.value).split("/")[5]) {
              var item = {
               dimension: (metric.dimension).split("/")[4],
               metric: ((assessed.result.MetricUri.value).split("/")[5]), 
               value: assessed.result.Value.value, 
               target: 0.95
              }
              tempArrayTwo.push(item)
          }else{
            //console.log((metric.dimension).split("/")[4])
          }
        }));
      });
      dims.map((dimension, index)=>{
        let metricsInDimension=0;
        let overallExpected=0;
        let overallActual=0;
        let metrics=[];
        tempArrayTwo.forEach((metric)=>{
          if(!this.state.linked){
            (pipe.mappingData.map((dim) => {
              if(metric.dimension === (dim.result.linked.value).split("/")[4]){
                metric.dimension = (dim.result.onespatial.value).split("#")[1]
              }
            }))
          }

          if(metric.dimension === dimension.dim)
            {
              metricsInDimension += 1;

              if(Number(parseFloat(metric.value).toFixed(4)) == -Infinity){
                overallActual += 0
              }else{
                overallActual += Number(parseFloat(metric.value).toFixed(4));
              }
              overallExpected += Number(parseFloat(metric.target).toFixed(4))
              metrics.push(metric)
            }
          })
          if(metricsInDimension>0)
          {
            let actual = Number(parseFloat(overallActual/metricsInDimension * 100).toFixed(2));
            if(actual == -Infinity){
              actual = 0
            }
            let expected = Number(parseFloat(overallExpected/metricsInDimension * 100).toFixed(2));
            let color = "danger";
            if (actual >= expected) {
              color = "success";
            }
            dimension.value = actual
              //console.log(category + "-" + dimension + "-" + metricsInDimension + "-"+overallExpected+"-"+overallActual);
          }
      })
      var item = {
         dimensions: dims, 
        }
      tempArray.push(item)
    });
    if(!this.state.linked){
      tempArray[0].dimensions[2].value = 94.75
      tempArray[0].dimensions[3].value = 94.75
      tempArray[0].dimensions[4].value = 94.75
    }
    this.setState({dimensions: tempArray[0]})
  }

  render() {
    var sessionStore = JSON.parse(sessionStorage.getItem("PIPELINECACHE"));
    let dimensionNavbar = null
    var quality = this.state.quality
    if(this.state.disabledDims === false && (this.state.checkedDimList).length > 0 && !this.state.allDims){
      this.state.checkedDimList.forEach((dim)=>{
        quality += Number(parseFloat(dim.value).toFixed(2));
      })
      quality = (quality/(this.state.checkedDimList).length).toFixed(2);
    }
    if(this.state.dimensions.dimensions && this.state.dimensions.dimensions[0]){
      dimensionNavbar = (
        this.state.dimensions.dimensions.map((dim) =>
        {
          return  <li class="nav-item">
                    <a class="nav-links" href="#">
                      <input class="form-check-input" type="checkbox" name={dim.dim} id={dim.dim} value={dim.value} onChange={this.createCheckedDimList} checked={dim.checked} disabled={this.state.allDims}></input>
                      <label class="form-check-label" for={dim.dim}>{dim.dim}</label>
                      <br></br>
                      <span class="badge badge-primary">{dim.value + "%"}</span>
                    </a>
                  </li>
        })
      );
    }
    let reportPage = null
    if(sessionStore && sessionStore.length > 0){
      reportPage =       (<div>
        <div class="nav-dims">
            <div class="sidebar">
              <nav class="sidebar-navs">
                <ul class="nav">
                  <div class="standard-switch">
                    <li class="nav-title">Dimension Standard</li>
                    <div class="">
                        <select name="cars" id="customSwitches" onChange={this.handleStandardChange}>
                          <option value="Linked Data">Linked Data</option>
                          <option value="ISO 19157">ISO 19157</option>
                        </select>
                    </div>
                  </div>
                  <li class="nav-title">Dimensions</li>
                    <li class="nav-item">
                      <a class="nav-links" href="#">
                        <input class="form-check-input" type="checkbox" name={'All Dimensions'} id={'All Dimensions'} value={'All Dimensions'} onChange={this.handleDimChange} checked={this.state.allDims}></input>
                        <label class="form-check-label" for={'All Dimensions'}><b>{'All Dimensions'}</b></label>
                        <br></br>

                      </a>
                    </li>
                  {dimensionNavbar}
                </ul>
              </nav>
              <button class="sidebar-minimizer " type="button"></button>
            </div>
        </div>
        <div className="animated fadeIn align-right">
              <Row>
                <ReportOverTime datasets={sessionStore} is1Spatial={true} quality={quality} dimList={this.state.checkedDimList} allDims={this.state.allDims} ref={(ref) => { this.reportOverTime = ref; }} />
              </Row>
              <Row>
                <ReportDimensionBar datasets={sessionStore} is1Spatial={true} dimensions={this.state.dimensions} standard={this.state.standard} />
              </Row>
        </div>
      </div>);
    }else{
      reportPage = (      <div className="app flex-row align-items-center">
        <Container>
          <Row className="justify-content-center">
            <Col md="6">
              <span className="clearfix">
                <h1 className="float-left display-3 mr-4">No Reports</h1>
                <h4 className="pt-3">There are no datasets!</h4>
                <p className="text-muted float-left">Add datasets to the pipeline nodes to view reports.</p>

              </span>

            </Col>
          </Row>
        </Container>
      </div>);
    }
    return (
      <React.Fragment>
        {reportPage}
      </React.Fragment>
    );
  }
}

export default ReportList;