//© 2019 Dublin City University, Trinity College Dublin. All rights reserved. This material may not be reproduced, displayed, modified or distributed without the express prior written permission of the copyright holder.
import React, { Component} from 'react';
import { Bar, Line } from 'react-chartjs-2';
import {
  Card,
  CardBody,
  CardHeader,
  ButtonDropdown,
  ButtonGroup,
  DropdownItem,
  DropdownMenu,
  DropdownToggle,
  Media,
  Modal,
  ModalHeader,
  ModalBody,
  ModalFooter,
  Button,
  FormGroup,
  Form,
  Label,
  Input,
} from 'reactstrap';
import { connect } from 'react-redux';
import { CustomTooltips } from '@coreui/coreui-plugin-chartjs-custom-tooltips';
import { getAllAssessmentDates } from '../../services/tripleStoreAPIs/sparql/getAllAssessmentDates';
import { getAssessmentQuality } from '../../services/tripleStoreAPIs/sparql/getAssessmentQuality';
import { read } from '../../services/tripleStoreAPIs/readFromTripleStore';
import LoadingSpinner from '../DataSetDetails/loading';
import LoadingFailed from '../DataSetDetails/loadingFailed';


const options = {
  tooltips: {
    enabled: false,
    custom: CustomTooltips
  },
  elements: {
    rectangle: {
      borderWidth: 4,
      borderSkipped: 'bottom'
    }
  },
  scales: {
    xAxes: [{
        barThickness: 20,
        scaleLabel: {
        display: true,
        labelString: "Time"

      }
    }],
    yAxes:[
      {
        ticks :{
          max: 100,
          min: 0,
          stepSize: 20
        },
        scaleLabel: {
        display: true,
        labelString: "Data Quality"

      }
      }
    ]
},
legend: {
    display: true
},

  maintainAspectRatio: false
}

class ReportOverTime extends Component{

  constructor(props) {
    super(props);
    this.state = {
      assessmentDates:[],
      assessmentQualities:[],
      qualityStatus :[],
      isloading : false,
      loadFailed :  false,
      dropdownOpen: false,
      popupOpen: false,
      tempLimit: sessionStorage.getItem("LIMIT"),
      quality: this.props.quality,
      colourList: ['rgb(0, 204, 255, 0.6)', 'rgb(255, 102, 0, 0.6)', 'rgb(0, 102, 0, 0.6)', 'rgb(51, 51, 255, 0.6)', 'rgb(153, 0, 51, 0.6)', 'rgb(255, 204, 0, 0.6)', 'rgb(0, 102, 204, 0.6)']
    };


  }

  togglePopup = () => {
      this.setState(prevState => ({
          popupOpen: !prevState.popupOpen
      }));
  }

  onChangeLimit = (event) =>
  {
    this.setState(
        { tempLimit: event.target.value }
    );
  }

  saveLimit = () =>
  {
    //console.log(this.state.tempLimit);
    sessionStorage.setItem("LIMIT",this.state.tempLimit );
    this.togglePopup();
  }

componentDidMount(){
  this.setDatasetData()
}

componentDidUpdate(prevProps, prevState){
  if(prevProps.quality !== this.props.quality){
    this.setDatasetData()
  }
}

setDatasetData = () => {

  this.setState({is1Spatial: this.props.is1Spatial})
    if (this.props.datasets.length>0) {
      this.setState({ isloading: true }, () => {
        this.setup(this.props.datasets[0])
        .then(()=>{
          let assessmentDate =[];
          let assessmentQuality = [];
          let assessmentColor =[];
          if(typeof this.props.datasets[0].historicAssessmentData!=="undefined")
          {
            if(this.props.datasets[0].historicAssessmentData.length>0)
            {
              if(this.state.is1Spatial){
                (this.props.datasets).forEach(function(dataset, index) {
                  if(index === 0){
                    assessmentDate = dataset.historicAssessmentData.map((result,index) => {
                      return ({index: index, result:result.dateAssessed});
                    });  
                  }
                });          
              }else{
                assessmentDate = this.props.datasets[0].historicAssessmentData.map((result,index) => {
                  return ({index: index, result:Object.keys(result)[0]});
                });
              }
            }
          }

          this.props.datasets[0].historicAssessmentData.forEach((date, index)=>{
            //console.log(date.result);
              let quality=0;
              let status="rgba(255,99,132,0.6)";
              let tempTotal=[];
              quality = Number(parseFloat(date.assessmentQuality).toFixed(4))*100
              if (quality <= this.props.datasets[0].expectedProgress) {
                status="rgba(0,255,0,0.6)";
              }

              assessmentQuality.push(
                {
                  index : index,
                  quality : Number.parseFloat(quality).toFixed(2)
                }
              );
              assessmentColor.push(
                {
                  index : index,
                  status : status
                }
              );

            });

            this.setState(
              {
                assessmentDates : assessmentDate,
                assessmentQualities : assessmentQuality,
                qualityStatus : assessmentColor,
                isloading: false
              }
            );
        });
      });
  }
}

 setup = async (props) =>
{
  let historicDateFromCache = await this.fetchHistoricDateFromCache(props.historicAssessmentData);
  let historicDateFromTripleStore = await this.fetchHistoricDateFromTripleStore(props.datasetPLD);
  for (const date of historicDateFromTripleStore)
  {
    await this.refreshCacheWithHistoricAssessmentData(historicDateFromCache, date, props.datasetID,props.datasetPLD);
  }

  return;
}


fetchHistoricDateFromCache = (historicAssessmentDataFromCache)=>
{
  return new Promise((resolve)=>{
      let cachedHistoricDates = [];
      if (typeof historicAssessmentDataFromCache!=="undefined")
      {
        historicAssessmentDataFromCache.forEach((assessmentData)=>{
          cachedHistoricDates.push(Object.keys(assessmentData));
        });
      }
      //console.log(cachedHistoricDates.flat());
      resolve(cachedHistoricDates.flat());

  });
}

fetchHistoricDateFromTripleStore = (datasetPLD) =>
{
    return new Promise((resolve)=>{
      let historicDates = [];
      read(getAllAssessmentDates(datasetPLD))
        .then((response) => {
          //console.log(response);
          if (response) {
            if (response.results.bindings.length > 0) {
              historicDates = response.results.bindings.map((result,index) => {
                return ({result:result.TimePeriod.value});
              });
            }
          }
          //console.log(historicDates);
          resolve(historicDates);
      });

    });
}

refreshCacheWithHistoricAssessmentData = (cachedHistoricDates, date, datasetID,datasetPLD) =>
{
    return new Promise((resolve)=>{

        //console.log(cachedHistoricDates.indexOf(date.result));
        if(cachedHistoricDates.indexOf(date.result) >= 0)
        {
          resolve();
        }
        else
        {
          //console.log("No");
          read(getAssessmentQuality(datasetPLD,date.result)).then((response) => {

              let assessmentResult = {
                [date.result] : [...response.results.bindings]
              }
              //console.log(assessmentResult);
              this.props.updateHistoricAssessmentDetailsToCache(assessmentResult,datasetID);
              resolve();
            }
          );
        }
    });
}



toggleAlertModel = () => {
    this.setState({
        loadFailed: false
    });
}

toggle = () => {
  this.setState({
    dropdownOpen: !this.state.dropdownOpen,
  });
}

dimensionColour = (value) => {
  let status="rgba(255,99,132)";
  if (value >= 95.00) {
    status="rgba(0,255,0)";
  }
  return status;
}

random_rgba = () => {
  var num = Math.round(0xffffff * Math.random());
  var r = num >> 16;
  var g = num >> 8 & 255;
  var b = num & 255;
  var colour = 'rgb(' + r + ', ' + g + ', ' + b + ',0.7)'
  return colour;
}

render (){

  console.log(this.state.assessmentDates)
    let mergedDateTime=[];
    let backgroundColour = this.state.qualityStatus.sort((a, b) => parseInt(a.index) - parseInt(b.index));
    let borderColor = this.state.qualityStatus.sort((a, b) => parseInt(a.index) - parseInt(b.index));
    let assessData = this.state.assessmentQualities.sort((a, b) => parseInt(a.index) - parseInt(b.index));

    let assessmentCount = this.state.assessmentDates.length;
    var dateTime = ''
    var dates = ''

    var time = ''
     dateTime = this.state.assessmentDates.map((date)=>{return((date.result));});
     console.log(dateTime.map((date)=>{return((date));}))
     dates = dateTime.map((date)=>{return((date).split(" ")[0]);});
     time = dateTime.map((date)=>{return((date).split(" ")[1]);});
    if(assessmentCount>sessionStorage.getItem("LIMIT"))
    {
      let startIndex=assessmentCount-sessionStorage.getItem("LIMIT");
      dateTime=dateTime.slice(startIndex);
      backgroundColour=backgroundColour.slice(startIndex);
      assessData=assessData.slice(startIndex);
    }
    let size = dateTime.length;
    let index=0;
    for (index=0; index<size;index++)
    {
      mergedDateTime.push([dates[index],time[index]]);
    }
  var sets = [
        {
          label: 'Prime2',
          backgroundColor: backgroundColour.map((status)=> {return 'rgb(0, 0, 255, 0.5)'}),
          //borderColor: backgroundColour.map((status)=> {return(status.status)}),
          data:  assessData.map((data)=>{return(data.quality)}) ,
        },
      ]
/*
      var fakedata1 = {
          label: 'MRDS',
          backgroundColor: 'rgb(226, 177, 60, 0.5)',
          //borderColor: 'rgb(255, 99,132)',
          data:  assessData.map((data)=>{return(data.quality)}) ,
        }
      var fakedata2 = {
          label: 'MRDS',
          backgroundColor: 'rgb(226, 177, 60, 0.5)',
          //borderColor: 'rgb(0, 255, 0)',
          data:  assessData.map((data)=>{return(data.quality)}) ,
        }
      var fakedata3 = {
          label: 'MRDS',
          backgroundColor: 'rgb(226, 177, 60, 0.5)',
          //borderColor: 'rgb(226, 177, 60,)',
          data:  [],
        }
    //sets[0].label.splice(1, 0, fakedata1.label);
    sets[0].backgroundColor.splice(1, 0, fakedata1.backgroundColor);
    //sets[0].borderColor.splice(1, 0, fakedata1.borderColor);
    sets[0].data.splice(1, 0, "38.45");

    //sets[0].label.splice(4, 0, fakedata2.label);
    sets[0].backgroundColor.splice(4, 0, fakedata2.backgroundColor);
    //sets[0].borderColor.splice(4, 0, fakedata2.borderColor);
    sets[0].data.splice(4, 0, "97.67");

    var fake1 = ["2019-05-29", "04:35:11"]
    var fake2 = ["2019-06-17", "12:05:52"]
    mergedDateTime.splice(1, 0, fake1);
    mergedDateTime.splice(4, 0, fake2);
    var fake3 = ["2019-05-29", "04:35:11"]
    var fake4 = ["2019-06-17", "12:05:52"]
    mergedDateTime.splice(3, 0, fake3);
    mergedDateTime.splice(5, 0, fake4);

    sets.push(fakedata3)
*/
  let bar = null
  if(this.props.allDims){
    bar = {
      labels: [...mergedDateTime],
      datasets: sets,

    };
  }else{
    var dimData = []
    for(var i = 0; i < this.props.dimList.length; i++){
      var data = 
        {
          label: '',
          backgroundColor: '',
          data: []
        }
      dimData.push(data)
    }
    for(var i = 0; i < dimData.length; i++){sessionStorage.getItem("LIMIT")
      if(this.props.dimList[i]){
        dimData[i].label = this.props.dimList[i].name
        dimData[i].backgroundColor = this.state.colourList[i]
        //dimData[i].borderColor = ['rgb(0, 0, 255, 0.5)', 'rgb(226, 177, 60)', 'rgb(0, 0, 255, 0.5)', 'rgb(0, 0, 255, 0.5)', 'rgb(226, 177, 60)', 'rgb(0, 0, 255, 0.5)', ]
        for(var e = 0; e < sessionStorage.getItem("LIMIT"); e++){
          dimData[i].data.push(this.props.dimList[i].value)
        }
        dimData[i].data = dimData[i].data.reverse()
      }
    }

    bar = {
      labels: [...mergedDateTime],
      datasets: dimData
    };
  }

  let model = (<Modal isOpen={true} className="modal-dialog-centered">
                  <ModalHeader style={{ margin: 'auto' }}>Configure Settings</ModalHeader>
                  <ModalBody>
                      <Form>
                          <FormGroup>
                              <Label>Dataset Name:</Label>
                               <Input type="text" name="limitInput" id="limitInput" ref="limitInput" value={this.state.tempLimit} onChange={this.onChangeLimit} />
                          </FormGroup>
                          </Form>
                      </ModalBody>
                      <ModalFooter>
                          <Button color="secondary" onClick={this.togglePopup}>Close</Button>
                          <Button color="primary" onClick={this.saveLimit}>Save</Button>{' '}
                      </ModalFooter>
                  </Modal>
                        );

return(
  <Card className="w-75 dim-bar-custom-height">
  {this.state.popupOpen?model:null}
    <CardHeader>
      Data Quality through Pipeline
      <ButtonGroup className="float-right">
        <ButtonDropdown id='card1' isOpen={this.state.dropdownOpen} toggle={this.toggle}>
          <DropdownToggle caret className="p-0" color="black">
            <Media className="icon-settings"></Media>
          </DropdownToggle>
          <DropdownMenu right>
            <DropdownItem onClick={this.togglePopup}>Config</DropdownItem>
          </DropdownMenu>
        </ButtonDropdown>
      </ButtonGroup>
    </CardHeader>
    <CardBody>

    {this.state.isloading ? <LoadingSpinner /> : null}
    {this.state.loadFailed ? <LoadingFailed clickHandler={this.toggleAlertModel} /> : null}
        <Bar data={bar} options={options} />
    </CardBody>
  </Card>
);

}

}

export default ReportOverTime;